# === Integer addition ===
assert 1 + 2 == 3, 'basic add'
assert 5 + 0 == 5, 'add zero'
assert 0 + 5 == 5, 'zero add'

# === Integer subtraction ===
assert 5 - 3 == 2, 'basic sub'
assert 5 - 0 == 5, 'sub zero'

# === Mixed int/float addition ===
assert 3 + 4.0 == 7.0, 'int add float'
assert 4.0 + 3 == 7.0, 'float add int'
assert -2 + 3.5 == 1.5, 'neg int add float'
assert 0 + 2.5 == 2.5, 'zero add float'
assert 2.5 + 0 == 2.5, 'float add zero'

# === Mixed int/float subtraction ===
assert 5 - 2.5 == 2.5, 'int sub float'
assert 5.5 - 2 == 3.5, 'float sub int'
assert -3 - 1.5 == -4.5, 'neg int sub float'
assert 1.5 - (-2) == 3.5, 'float sub neg int'

# === Float subtraction ===
assert 5.5 - 2.5 == 3.0, 'float sub float'
assert 0.0 - 1.5 == -1.5, 'zero sub float'

# === Integer modulo ===
assert 10 % 3 == 1, 'basic mod'
assert 3 % 10 == 3, 'mod larger divisor'
assert 9 % 3 == 0, 'mod zero result'

# === Augmented assignment (+=) ===
x = 5
x += 3
assert x == 8, 'basic iadd'

# === Integer repr/str ===
assert repr(42) == '42', 'int repr'
assert str(42) == '42', 'int str'

# === Float repr/str ===
assert repr(2.5) == '2.5', 'float repr'
assert str(2.5) == '2.5', 'float str'

# === Integer multiplication ===
assert 3 * 4 == 12, 'basic int mult'
assert 5 * 0 == 0, 'mult by zero'
assert 0 * 5 == 0, 'zero mult'
assert -3 * 4 == -12, 'negative mult'
assert 3 * -4 == -12, 'mult negative'
assert -3 * -4 == 12, 'neg mult neg'

# === Float multiplication ===
assert 3.0 * 4.0 == 12.0, 'float mult'
assert 2.5 * 2.0 == 5.0, 'float mult 2'

# === Mixed int/float multiplication ===
assert 3 * 4.0 == 12.0, 'int mult float'
assert 4.0 * 3 == 12.0, 'float mult int'

# === True division (always returns float) ===
assert 6 / 2 == 3.0, 'int div exact'
assert 7 / 2 == 3.5, 'int div remainder'
assert 1 / 4 == 0.25, 'int div fraction'
assert 6.0 / 2.0 == 3.0, 'float div'
assert 7 / 2.0 == 3.5, 'int div float'
assert 7.0 / 2 == 3.5, 'float div int'
assert -7 / 2 == -3.5, 'neg div'

# === Floor division ===
assert 7 // 2 == 3, 'int floor div'
assert 6 // 2 == 3, 'int floor div exact'
assert -7 // 2 == -4, 'neg floor div rounds down'
assert 7 // -2 == -4, 'floor div neg rounds down'
assert -7 // -2 == 3, 'neg floor div neg'
assert 7.0 // 2.0 == 3.0, 'float floor div'
assert 7 // 2.0 == 3.0, 'int floor div float'
assert 7.0 // 2 == 3.0, 'float floor div int'
assert -7.0 // 2.0 == -4.0, 'neg float floor div'

# === Power (exponentiation) ===
assert 2**3 == 8, 'int pow'
assert 2**10 == 1024, 'int pow large'
assert 2**0 == 1, 'pow zero'
assert (-2) ** 3 == -8, 'neg base pow'
assert (-2) ** 2 == 4, 'neg base even pow'
assert 2**-1 == 0.5, 'pow neg returns float'
assert 2**-2 == 0.25, 'pow neg 2'
assert 4.0**2.0 == 16.0, 'float pow'
assert 4**0.5 == 2.0, 'sqrt via pow'
assert 8 ** (1 / 3) == 2.0, 'cube root via pow'
assert 2.0**3 == 8.0, 'float pow int'

# === Augmented assignment operators ===
# *=
x = 5
x *= 3
assert x == 15, 'imult'

# /=
x = 10
x /= 4
assert x == 2.5, 'idiv'

# //=
x = 10
x //= 3
assert x == 3, 'ifloordiv'

# **=
x = 2
x **= 4
assert x == 16, 'ipow'

# -=
x = 10
x -= 3
assert x == 7, 'isub'

# %=
x = 10
x %= 3
assert x == 1, 'imod'

# === Bool arithmetic (True=1, False=0) ===
# Bool multiplication
assert True * 3 == 3, 'bool mult int'
assert False * 5 == 0, 'false mult int'
assert 3 * True == 3, 'int mult bool'
assert 3 * False == 0, 'int mult false'
assert True * True == 1, 'bool mult bool'
assert True * False == 0, 'bool mult false'
assert True * 2.5 == 2.5, 'bool mult float'
assert 2.5 * True == 2.5, 'float mult bool'

# Bool division
assert True / 2 == 0.5, 'bool div int'
assert False / 2 == 0.0, 'false div int'
assert 4 / True == 4.0, 'int div bool'
assert True / True == 1.0, 'bool div bool'
assert True / 2.0 == 0.5, 'bool div float'
assert 4.0 / True == 4.0, 'float div bool'

# Bool floor division
assert True // 2 == 0, 'bool floordiv int'
assert False // 2 == 0, 'false floordiv int'
assert 5 // True == 5, 'int floordiv bool'
assert True // True == 1, 'bool floordiv bool'
assert True // 2.0 == 0.0, 'bool floordiv float'
assert 5.5 // True == 5.0, 'float floordiv bool'

# Bool power
assert True**3 == 1, 'bool pow int'
assert False**3 == 0, 'false pow int'
assert 2**True == 2, 'int pow bool true'
assert 2**False == 1, 'int pow bool false'
assert True**True == 1, 'bool pow bool'
assert False**False == 1, 'false pow false'
assert True**2.0 == 1.0, 'bool pow float'
assert 2.0**True == 2.0, 'float pow bool true'
assert 2.0**False == 1.0, 'float pow bool false'

# === Unary positive (no-op for numbers, converts bools to int) ===
assert +5 == 5, 'unary pos int'
assert +(-3) == -3, 'unary pos neg int'
assert +0 == 0, 'unary pos zero'
assert +3.14 == 3.14, 'unary pos float'
assert +(-2.5) == -2.5, 'unary pos neg float'
assert +0.0 == 0.0, 'unary pos zero float'
assert +True == 1, 'unary pos true'
assert +False == 0, 'unary pos false'
# Verify +bool returns int type, not bool
assert type(+True) == int, 'unary pos true returns int type'
assert type(+False) == int, 'unary pos false returns int type'

# === Unary negative ===
assert -5 == -5, 'unary neg int'
assert -(-3) == 3, 'unary neg neg int'
assert -0 == 0, 'unary neg zero'
assert -3.14 == -3.14, 'unary neg float'
assert -(-2.5) == 2.5, 'unary neg neg float'
assert -True == -1, 'unary neg true'
assert repr(-True) == '-1', 'unary neg true repr'
assert -False == 0, 'unary neg false'
assert repr(-False) == '0', 'unary neg false repr'

# === Unary invert (bitwise NOT) ===
assert ~0 == -1, 'unary invert zero'
assert ~1 == -2, 'unary invert one'
assert ~(-1) == 0, 'unary invert neg one'
assert ~True == -2, 'unary invert true'
assert repr(~True) == '-2', 'unary invert true repr'
assert ~False == -1, 'unary invert false'
assert repr(~False) == '-1', 'unary invert false repr'

assert int('123') == 123, 'int conversion from string'
assert int('  123  ') == 123, 'int conversion from string trim'
assert int('1_234 ') == 1234, 'int conversion from string'
assert int('ff', 16) == 255, 'int conversion with explicit base'
assert int('ff', base=16) == 255, 'int conversion with keyword base'
assert int('0xff', 16) == 255, 'int conversion with base prefix'
assert int('0xff', 0) == 255, 'int conversion with autodetect base'

try:
    int('abc')
except ValueError as e:
    assert str(e) == "invalid literal for int() with base 10: 'abc'", f'got err: {e}'
else:
    raise AssertionError('int conversion from string should fail')

try:
    int(' ')
except ValueError as e:
    assert str(e) == "invalid literal for int() with base 10: ' '", f'got err: {e}'
else:
    raise AssertionError('int conversion from string should fail')

try:
    int('a\tbc')
except ValueError as e:
    assert str(e) == "invalid literal for int() with base 10: 'a\\tbc'", f'got err: {e}'
else:
    raise AssertionError('int conversion from string should fail')

try:
    int(10, 2)
except TypeError as e:
    assert str(e) == "int() can't convert non-string with explicit base", f'got err: {e}'
else:
    raise AssertionError('int non-string with explicit base should fail')



# Test that heap values are properly cleaned up when missing required arg error occurs
def f(a, b, c):
    return a


# The list [1, 2, 3] should be cleaned up when the error occurs
# because 'c' is missing
f([1, 2, 3], [4, 5])
# Raise=TypeError("f() missing 1 required positional argument: 'c'")



# Tests for collections.abc public API and core compatibility behavior

# === Module import ===
import collections.abc as cabc

# === Public API presence ===
expected_public_names = [
    'ABCMeta',
    'AsyncGenerator',
    'AsyncIterable',
    'AsyncIterator',
    'Awaitable',
    'Buffer',
    'ByteString',
    'Callable',
    'Collection',
    'Container',
    'Coroutine',
    'EllipsisType',
    'FunctionType',
    'Generator',
    'GenericAlias',
    'Hashable',
    'ItemsView',
    'Iterable',
    'Iterator',
    'KeysView',
    'Mapping',
    'MappingView',
    'MutableMapping',
    'MutableSequence',
    'MutableSet',
    'Reversible',
    'Sequence',
    'Set',
    'Sized',
    'ValuesView',
    'abstractmethod',
    'async_generator',
    'bytearray_iterator',
    'bytes_iterator',
    'coroutine',
    'dict_itemiterator',
    'dict_items',
    'dict_keyiterator',
    'dict_keys',
    'dict_valueiterator',
    'dict_values',
    'framelocalsproxy',
    'generator',
    'list_iterator',
    'list_reverseiterator',
    'longrange_iterator',
    'mappingproxy',
    'range_iterator',
    'set_iterator',
    'str_iterator',
    'sys',
    'tuple_iterator',
    'zip_iterator',
]

for name in expected_public_names:
    assert hasattr(cabc, name), f'collections.abc should export {name}'

# === Module-level bindings ===
assert cabc.sys is not None, 'collections.abc.sys should be available'
assert hasattr(cabc.sys, 'platform'), 'collections.abc.sys should expose sys-like attributes'


# === Decorator behavior ===
def sample_function():
    return 42


decorated = cabc.abstractmethod(sample_function)
assert decorated is sample_function, 'abstractmethod should return the original function'
assert getattr(decorated, '__isabstractmethod__', False) is True, 'abstractmethod should mark function as abstract'

try:
    cabc.abstractmethod()
except TypeError:
    pass
else:
    assert False, 'abstractmethod should raise TypeError when called without arguments'

# === Core type aliases ===
assert cabc.EllipsisType is type(...), 'EllipsisType should match ellipsis literal type'
assert cabc.FunctionType is type(sample_function), 'FunctionType should match function type'
assert isinstance(cabc.GenericAlias, type), 'GenericAlias should be a type-like object'
assert isinstance(cabc.mappingproxy, type), 'mappingproxy should be a type-like object'
assert isinstance(cabc.generator, type), 'generator should be a type-like object'
assert isinstance(cabc.coroutine, type), 'coroutine should be a type-like object'
assert isinstance(cabc.async_generator, type), 'async_generator should be a type-like object'
assert isinstance(cabc.framelocalsproxy, type), 'framelocalsproxy should be a type-like object'

# === Concrete runtime checks ===
d = {'a': 1, 'b': 2}
assert isinstance(d.keys(), cabc.dict_keys), 'dict_keys should match dict.keys() type'
assert isinstance(d.values(), cabc.dict_values), 'dict_values should match dict.values() type'
assert isinstance(d.items(), cabc.dict_items), 'dict_items should match dict.items() type'

assert isinstance(iter([1]), cabc.list_iterator), 'list_iterator should match iter(list)'
assert isinstance(iter((1,)), cabc.tuple_iterator), 'tuple_iterator should match iter(tuple)'
assert isinstance(iter(range(3)), cabc.range_iterator), 'range_iterator should match iter(range)'
assert isinstance(iter({'k': 1}.keys()), cabc.dict_keyiterator), 'dict_keyiterator should match iter(dict.keys())'
assert isinstance(iter({'k': 1}.values()), cabc.dict_valueiterator), (
    'dict_valueiterator should match iter(dict.values())'
)
assert isinstance(iter({'k': 1}.items()), cabc.dict_itemiterator), 'dict_itemiterator should match iter(dict.items())'
assert isinstance(iter({1, 2}), cabc.set_iterator), 'set_iterator should match iter(set)'
assert isinstance(iter('ab'), cabc.str_iterator), 'str_iterator should match iter(str)'
assert isinstance(iter(b'ab'), cabc.bytes_iterator), 'bytes_iterator should match iter(bytes)'
assert isinstance(iter(bytearray(b'ab')), cabc.bytearray_iterator), 'bytearray_iterator should match iter(bytearray)'
assert isinstance(cabc.list_reverseiterator, type), 'list_reverseiterator should be a type-like object'
assert isinstance(cabc.longrange_iterator, type), 'longrange_iterator should be a type-like object'
assert isinstance(cabc.zip_iterator, type), 'zip_iterator should be a type-like object'


# === Generator instance type ===
def make_generator():
    yield 1


assert isinstance(make_generator(), cabc.generator), 'generator should match generator instance type'



# === map() basic ===
# Test map() with builtin functions
result = list(map(len, ['a', 'bb', 'ccc']))
assert result == [1, 2, 3], 'map with len'

# empty iterable
assert list(map(len, [])) == [], 'map empty'

# === map() with type constructors ===
result = list(map(int, ['1', '2', '3']))
assert result == [1, 2, 3], 'map with int constructor'

result = list(map(str, [1, 2, 3]))
assert result == ['1', '2', '3'], 'map with str constructor'

# === map() with multiple iterables ===
# Note: lambda functions not yet supported in map() due to VM frame requirements
# For now, test with builtins that take multiple args (like max, min in theory)
# Actually max/min don't work well with map since they return single values

# === map() stops at shortest iterable ===
result = list(map(len, ['a', 'bb', 'ccc', 'dddd', 'eeeee']))
assert result == [1, 2, 3, 4, 5], 'map stops correctly'



# === Method resolution on instances ===
class Animal:
    def __init__(self, name):
        self.name = name

    def speak(self):
        return f'{self.name} speaks'


a = Animal('dog')
assert a.speak() == 'dog speaks', 'instance method call'


# === Inherited method ===
class Dog(Animal):
    pass


d = Dog('rex')
assert d.speak() == 'rex speaks', 'inherited method'


# === Method that calls another method ===
class Cat(Animal):
    def greet(self):
        return f'Hi, {self.speak()}'


c = Cat('whiskers')
assert c.greet() == 'Hi, whiskers speaks', 'method calling method'


# === Instance attribute shadows class method ===
class Test:
    def method(self):
        return 'class method'


t = Test()
assert t.method() == 'class method', 'method before shadowing'
# Setting instance attribute would shadow the class method
# t.method = 'instance attribute'  # This would shadow


# === __set_name__ descriptor protocol ===
class Descriptor:
    def __set_name__(self, owner, name):
        self.owner = owner
        self.name = name

    def __get__(self, obj, objtype=None):
        return self.name


class MyClass:
    x = Descriptor()
    y = Descriptor()


assert MyClass.x == 'x', '__set_name__ sets name for x'
assert MyClass.y == 'y', '__set_name__ sets name for y'



# === Basic chain comparisons ===
assert (1 < 2 < 3) == True, 'ascending chain'
assert (1 < 3 < 2) == False, 'fails at second comparison'
assert (3 < 2 < 1) == False, 'fails at first comparison'
assert (1 <= 2 <= 2) == True, 'with equality'
assert 1 <= 2 <= 2, 'with equality'
assert 1 <= 2 <= 2 <= 3, 'chained with equality'

# === Mixed operators ===
assert (1 < 2 <= 2 < 3) == True, 'mixed lt and le'
assert (1 == 1 == 1) == True, 'triple equality'
assert (1 != 2 != 1) == True, 'not-equal chain (not transitive)'

# === Longer chains ===
assert (1 < 2 < 3 < 4 < 5) == True, '5-way ascending'
assert (1 < 2 < 3 < 2 < 5) == False, 'fails in middle'

# === With variables and expressions ===
x = 5
assert (1 < x < 10) == True, 'variable in chain'
assert (0 < x - 3 < x < x + 1) == True, 'expressions'


# === Short-circuit evaluation ===
def test_short_circuit():
    calls = []

    def a():
        calls.append('a')
        return 1

    def b():
        calls.append('b')
        return 0  # This will make first comparison fail

    def c():
        calls.append('c')
        return 2

    # Test: first comparison fails, c() should not be called
    result = a() < b() < c()  # 1 < 0 is False, c() should not be called
    assert result == False, 'short circuit result'
    assert calls == ['a', 'b'], 'c not called due to short circuit'


test_short_circuit()


# === Single evaluation of intermediate values ===
def test_single_eval():
    count = 0

    def middle():
        nonlocal count
        count += 1
        return 5

    result = 1 < middle() < 10
    assert result == True, 'chain result'
    assert count == 1, 'middle() called exactly once'


test_single_eval()

# === Identity comparisons ===
a = [1]
b = a
c = a
assert (a is b is c) == True, 'is chain same object'

# === Containment checks ===
assert (1 in [1, 2] in [[1, 2], [3]]) == True, 'in chain'


# === Verify no namespace pollution ===
# Note: The old implementation used _chain_cmp_N variables which would leak.
# The new stack-based implementation doesn't create any intermediate variables.
# We can't easily test for namespace pollution without dir(), so we just verify
# that chain comparisons work correctly (covered by tests above).



import traceback


# === Module API ===
for name in [
    'format_exc',
    'format_exception',
    'format_tb',
    'format_stack',
    'print_exc',
    'print_exception',
    'print_tb',
    'print_stack',
    'extract_tb',
    'extract_stack',
    'format_list',
    'FrameSummary',
    'StackSummary',
    'TracebackException',
]:
    assert hasattr(traceback, name), f'missing traceback.{name}'


# === FrameSummary ===
frame = traceback.FrameSummary('demo.py', 10, 'func', line='x = 1')
assert frame.filename == 'demo.py', 'FrameSummary.filename'
assert frame.lineno == 10, 'FrameSummary.lineno'
assert frame.name == 'func', 'FrameSummary.name'
assert frame.line == 'x = 1', 'FrameSummary.line'


# === format_list ===
formatted_list = traceback.format_list([('demo.py', 10, 'func', 'x = 1')])
assert formatted_list == ['  File "demo.py", line 10, in func\n    x = 1\n'], 'format_list tuple input'


# === format helpers ===
assert traceback.format_exc() == 'NoneType: None\n', 'format_exc with no active exception'
assert traceback.format_tb(None) == [], 'format_tb(None)'

formatted_exception = traceback.format_exception(ValueError, ValueError('bad'), None)
assert isinstance(formatted_exception, list), 'format_exception returns list'
assert formatted_exception[-1] == 'ValueError: bad\n', 'format_exception line'

stack_lines = traceback.format_stack(limit=0)
assert isinstance(stack_lines, list), 'format_stack returns list'


# === extraction helpers ===
summary_from_tb = traceback.extract_tb(None)
assert hasattr(summary_from_tb, 'format'), 'extract_tb returns StackSummary-like object'
assert summary_from_tb.format() == [], 'extract_tb(None) empty'

summary_from_extract = traceback.StackSummary.extract([])
assert hasattr(summary_from_extract, 'format'), 'StackSummary.extract returns object with format'
assert summary_from_extract.format() == [], 'StackSummary.extract([]) empty'


# === TracebackException ===
tb_exc = traceback.TracebackException(ValueError, ValueError('bad'), None)
assert list(tb_exc.format_exception_only()) == ['ValueError: bad\n'], 'TracebackException.format_exception_only'
assert list(tb_exc.format())[-1] == 'ValueError: bad\n', 'TracebackException.format'


# === print helpers ===
traceback.print_exception(ValueError, ValueError('bad'), None, file=None)



# === Fraction imports ===
from fractions import Fraction

# === Basic construction ===
# Fraction() -> 0/1
f = Fraction()
assert f.numerator == 0, 'Fraction() numerator is 0'
assert f.denominator == 1, 'Fraction() denominator is 1'
assert str(f) == '0', 'Fraction() str is 0'

# Fraction(n) -> n/1
f = Fraction(5)
assert f.numerator == 5, 'Fraction(5) numerator is 5'
assert f.denominator == 1, 'Fraction(5) denominator is 1'
assert str(f) == '5', 'Fraction(5) str is 5'

# Fraction(n, d) -> normalized
f = Fraction(4, 8)
assert f.numerator == 1, 'Fraction(4, 8) numerator is 1'
assert f.denominator == 2, 'Fraction(4, 8) denominator is 2'
assert str(f) == '1/2', 'Fraction(4, 8) str is 1/2'

# Negative fractions
f = Fraction(-3, 6)
assert f.numerator == -1, 'Fraction(-3, 6) numerator is -1'
assert f.denominator == 2, 'Fraction(-3, 6) denominator is 2'

f = Fraction(3, -6)
assert f.numerator == -1, 'Fraction(3, -6) numerator is -1'
assert f.denominator == 2, 'Fraction(3, -6) denominator is 2'

f = Fraction(-3, -6)
assert f.numerator == 1, 'Fraction(-3, -6) numerator is 1'
assert f.denominator == 2, 'Fraction(-3, -6) denominator is 2'

# Zero
f = Fraction(0, 5)
assert f.numerator == 0, 'Fraction(0, 5) numerator is 0'
assert f.denominator == 1, 'Fraction(0, 5) denominator is 1'

# === From string ===
f = Fraction('3/7')
assert f.numerator == 3, 'Fraction("3/7") numerator is 3'
assert f.denominator == 7, 'Fraction("3/7") denominator is 7'

f = Fraction('  -3/7  ')
assert f.numerator == -3, 'Fraction("  -3/7  ") numerator is -3'
assert f.denominator == 7, 'Fraction("  -3/7  ") denominator is 7'

f = Fraction('1.5')
assert f.numerator == 3, 'Fraction("1.5") numerator is 3'
assert f.denominator == 2, 'Fraction("1.5") denominator is 2'

# === From float ===
f = Fraction(0.5)
assert f.numerator == 1, 'Fraction(0.5) numerator is 1'
assert f.denominator == 2, 'Fraction(0.5) denominator is 2'

f = Fraction(-0.75)
assert f.numerator == -3, 'Fraction(-0.75) numerator is -3'
assert f.denominator == 4, 'Fraction(-0.75) denominator is 4'

# === Properties ===
f = Fraction(3, 4)
assert f.numerator == 3, 'numerator property'
assert f.denominator == 4, 'denominator property'

# === String representation ===
f = Fraction(3, 4)
assert str(f) == '3/4', 'str(Fraction(3, 4))'
assert repr(f) == 'Fraction(3, 4)', 'repr(Fraction(3, 4))'

f = Fraction(5)
assert str(f) == '5', 'str(Fraction(5))'
assert repr(f) == 'Fraction(5, 1)', 'repr(Fraction(5))'

# === Arithmetic operations ===
a = Fraction(1, 2)
b = Fraction(1, 3)

# Addition
result = a + b
assert result.numerator == 5, '1/2 + 1/3 = 5/6'
assert result.denominator == 6, '1/2 + 1/3 = 5/6'

# Subtraction
result = a - b
assert result.numerator == 1, '1/2 - 1/3 = 1/6'
assert result.denominator == 6, '1/2 - 1/3 = 1/6'

# Multiplication
result = a * b
assert result.numerator == 1, '1/2 * 1/3 = 1/6'
assert result.denominator == 6, '1/2 * 1/3 = 1/6'

# Division
result = a / b
assert result.numerator == 3, '1/2 / 1/3 = 3/2'
assert result.denominator == 2, '1/2 / 1/3 = 3/2'

# Floor division
result = a // b
assert result == 1, '1/2 // 1/3 = 1'

# Modulo
result = a % b
assert result.numerator == 1, '1/2 % 1/3 = 1/6'
assert result.denominator == 6, '1/2 % 1/3 = 1/6'

# Power
result = a ** 3
assert result.numerator == 1, '(1/2) ** 3 = 1/8'
assert result.denominator == 8, '(1/2) ** 3 = 1/8'

# Negation
result = -a
assert result.numerator == -1, '-(1/2) = -1/2'
assert result.denominator == 2, '-(1/2) = -1/2'

# Absolute value
result = abs(Fraction(-3, 4))
assert result.numerator == 3, 'abs(-3/4) = 3/4'
assert result.denominator == 4, 'abs(-3/4) = 3/4'

# === Comparison ===
a = Fraction(1, 2)
b = Fraction(2, 4)
c = Fraction(1, 3)

assert a == b, '1/2 == 2/4'
assert a != c, '1/2 != 1/3'
assert a > c, '1/2 > 1/3'
assert c < a, '1/3 < 1/2'
assert a >= b, '1/2 >= 2/4'
assert c <= a, '1/3 <= 1/2'

# === limit_denominator ===
f = Fraction(355, 113)
result = f.limit_denominator()
assert result.numerator == 355, 'limit_denominator keeps 355/113 when already close'
assert result.denominator == 113, 'limit_denominator keeps 355/113 when already close'

result = f.limit_denominator(10)
assert result.numerator == 22, '355/113 with max_denominator=10 is 22/7'
assert result.denominator == 7, '355/113 with max_denominator=10 is 22/7'

result = f.limit_denominator(100)
assert result.numerator == 311, '355/113 with max_denominator=100 is 311/99'
assert result.denominator == 99, '355/113 with max_denominator=100 is 311/99'

# === as_integer_ratio ===
f = Fraction(3, 4)
n, d = f.as_integer_ratio()
assert n == 3, 'as_integer_ratio returns (3, 4) for 3/4'
assert d == 4, 'as_integer_ratio returns (3, 4) for 3/4'

# === Boolean ===
assert bool(Fraction(1, 2)) is True, 'bool(Fraction(1, 2)) is True'
assert bool(Fraction(0)) is False, 'bool(Fraction(0)) is False'
assert bool(Fraction(-1, 2)) is True, 'bool(Fraction(-1, 2)) is True'

# === float() conversion ===
f = Fraction(355, 113)
assert abs(float(f) - 3.14159292) < 0.0001, 'float(Fraction(355, 113)) ~ pi'
assert float(Fraction(1, 3)) == 1/3, 'float(Fraction(1, 3)) == 1/3'
assert float(Fraction(7, 2)) == 3.5, 'float(Fraction(7, 2)) == 3.5'
assert float(Fraction(0)) == 0.0, 'float(Fraction(0)) == 0.0'
assert float(Fraction(-3, 4)) == -0.75, 'float(Fraction(-3, 4)) == -0.75'
assert float(Fraction(1)) == 1.0, 'float(Fraction(1)) == 1.0'

# === int() conversion ===
assert int(Fraction(7, 2)) == 3, 'int(Fraction(7, 2)) == 3'
assert int(Fraction(-7, 2)) == -3, 'int(Fraction(-7, 2)) == -3'
assert int(Fraction(0)) == 0, 'int(Fraction(0)) == 0'
assert int(Fraction(5, 1)) == 5, 'int(Fraction(5, 1)) == 5'



# === type() function ===
assert type(1) == int, 'type(int) returns int'
assert type(1.5) == float, 'type(float) returns float'
assert type(True) == bool, 'type(bool) returns bool'
assert type('hello') == str, 'type(str) returns str'
assert type([1, 2]) == list, 'type(list) returns list'
assert type((1, 2)) == tuple, 'type(tuple) returns tuple'
assert type({1: 2}) == dict, 'type(dict) returns dict'
assert type(b'hi') == bytes, 'type(bytes) returns bytes'
assert type(None) == type(None), 'type(None) is consistent'

# === type() inequality ===
assert type(1) != str, 'int type != str'
assert type([]) != tuple, 'list type != tuple'
assert type({}) != list, 'dict type != list'
assert type(1) != float, 'int type != float'

# === type repr ===
assert repr(int) == "<class 'int'>", 'int type repr'
assert repr(float) == "<class 'float'>", 'float type repr'
assert repr(bool) == "<class 'bool'>", 'bool type repr'
assert repr(str) == "<class 'str'>", 'str type repr'
assert repr(list) == "<class 'list'>", 'list type repr'
assert repr(tuple) == "<class 'tuple'>", 'tuple type repr'
assert repr(dict) == "<class 'dict'>", 'dict type repr'
assert repr(bytes) == "<class 'bytes'>", 'bytes type repr'

# === type identity ===
assert int is int, 'int is int'
assert str is str, 'str is str'
assert list is list, 'list is list'
assert type(1) is int, 'type(1) is int'
assert type('') is str, 'type str is str'
assert type([]) is list, 'type([]) is list'

# === list() constructor ===
assert list() == [], 'list() empty'
assert list([1, 2, 3]) == [1, 2, 3], 'list(list) copy'
assert list((1, 2, 3)) == [1, 2, 3], 'list(tuple) convert'
assert list(range(3)) == [0, 1, 2], 'list(range) convert'
assert list('abc') == ['a', 'b', 'c'], 'list(str) split chars'
assert list('') == [], 'list empty str'

# list copy is independent
orig = [1, 2, 3]
copy = list(orig)
copy.append(4)
assert orig == [1, 2, 3], 'list copy is independent'
assert copy == [1, 2, 3, 4], 'list copy modified'

# === tuple() constructor ===
assert tuple() == (), 'tuple() empty'
assert tuple([1, 2, 3]) == (1, 2, 3), 'tuple(list) convert'
assert tuple((1, 2)) == (1, 2), 'tuple(tuple) copy'
assert tuple(range(3)) == (0, 1, 2), 'tuple(range) convert'
assert tuple('ab') == ('a', 'b'), 'tuple(str) split chars'
assert tuple('') == (), 'tuple empty str'

# === dict() constructor ===
assert dict() == {}, 'dict() empty'
assert dict({1: 2}) == {1: 2}, 'dict(dict) copy'
assert dict({'a': 1, 'b': 2}) == {'a': 1, 'b': 2}, 'dict(dict) multiple keys'

# dict copy is independent
orig_dict = {1: 2}
copy_dict = dict(orig_dict)
copy_dict[3] = 4
assert orig_dict == {1: 2}, 'dict copy is independent'
assert copy_dict == {1: 2, 3: 4}, 'dict copy modified'

# === str() constructor ===
assert str() == '', 'str() empty'
assert str(123) == '123', 'str(int)'
assert str(-42) == '-42', 'str(negative int)'
assert str(0) == '0', 'str(zero)'
assert str(1.5) == '1.5', 'str(float)'
assert str(True) == 'True', 'str(bool True)'
assert str(False) == 'False', 'str(bool False)'
assert str(None) == 'None', 'str(None)'
assert str([1, 2]) == '[1, 2]', 'str(list)'
assert str((1, 2)) == '(1, 2)', 'str(tuple)'
assert str({1: 2}) == '{1: 2}', 'str(dict)'
assert str('hello') == 'hello', 'str(str)'
assert str(b'hi') == "b'hi'", 'str(bytes)'

# === bytes() constructor ===
assert bytes() == b'', 'bytes() empty'
assert bytes(3) == b'\x00\x00\x00', 'bytes(int) zero-filled'
assert bytes(0) == b'', 'bytes(0) empty'
assert bytes(b'hi') == b'hi', 'bytes(bytes) copy'
assert bytes([65, 66, 67]) == b'ABC', 'bytes(list_of_ints)'
assert bytes(range(3)) == b'\x00\x01\x02', 'bytes(range)'

# === int() constructor ===
assert int() == 0, 'int() default'
assert int(42) == 42, 'int(int)'
assert int(-5) == -5, 'int(negative int)'
assert int(3.7) == 3, 'int(float) truncates down'
assert int(-3.7) == -3, 'int(negative float) truncates toward zero'
assert int(3.0) == 3, 'int(whole float)'
assert int(True) == 1, 'int(True)'
assert int(False) == 0, 'int(False)'

# int() with extreme float values (should clamp to i64 range in Ouros)
# Note: Python uses arbitrary precision; Ouros clamps to i64
assert isinstance(int(1e18), int), 'int(large float) returns int'
assert isinstance(int(-1e18), int), 'int(large negative float) returns int'
assert int(0.0) == 0, 'int(0.0) is zero'
assert int(-0.0) == 0, 'int(-0.0) is zero'
assert int(0.9) == 0, 'int(0.9) truncates to 0'
assert int(-0.9) == 0, 'int(-0.9) truncates to 0'

# === float() constructor ===
assert float() == 0.0, 'float() default'
assert float(42) == 42.0, 'float(int)'
assert float(-5) == -5.0, 'float(negative int)'
assert float(3.14) == 3.14, 'float(float)'
assert float(True) == 1.0, 'float(True)'
assert float(False) == 0.0, 'float(False)'

# === bool() constructor ===
assert bool() == False, 'bool() default'
assert bool(0) == False, 'bool(0)'
assert bool(1) == True, 'bool(1)'
assert bool(-1) == True, 'bool(-1)'
assert bool(0.0) == False, 'bool(0.0)'
assert bool(1.5) == True, 'bool(1.5)'
assert bool('') == False, 'bool empty str'
assert bool('x') == True, 'bool non-empty str'
assert bool([]) == False, 'bool empty list'
assert bool([1]) == True, 'bool non-empty list'
assert bool(()) == False, 'bool empty tuple'
assert bool((1,)) == True, 'bool non-empty tuple'
assert bool({}) == False, 'bool empty dict'
assert bool({1: 2}) == True, 'bool non-empty dict'
assert bool(None) == False, 'bool(None)'

# === isinstance() ===
assert isinstance(1, int), 'isinstance int'
assert isinstance(1.5, float), 'isinstance float'
assert isinstance(True, bool), 'isinstance bool'
assert isinstance('hello', str), 'isinstance str'
assert isinstance([1, 2], list), 'isinstance list'
assert isinstance((1, 2), tuple), 'isinstance tuple'
assert isinstance({1: 2}, dict), 'isinstance dict'
assert isinstance(b'hi', bytes), 'isinstance bytes'

# isinstance negative cases
assert not isinstance(1, str), 'isinstance int not str'
assert not isinstance('x', int), 'isinstance str not int'
assert not isinstance([], dict), 'isinstance list not dict'

# isinstance with tuple of types
assert isinstance(1, (int, str)), 'isinstance tuple match first'
assert isinstance('x', (int, str)), 'isinstance tuple match second'
assert not isinstance([], (int, str)), 'isinstance tuple no match'
assert isinstance(1, (str, float, int)), 'isinstance tuple match third'

# bool is subtype of int
assert isinstance(True, int), 'bool is instance of int'
assert isinstance(False, int), 'False is instance of int'
assert isinstance(True, (int, str)), 'bool matches int in tuple'

# isinstance with exception types
err = ValueError('test')
assert isinstance(err, ValueError), 'isinstance exception'
assert isinstance(err, Exception), 'isinstance exception base type'
assert not isinstance(err, TypeError), 'isinstance exception wrong type'
assert isinstance(err, (ValueError, TypeError)), 'isinstance exception tuple'

# isinstance with nested tuples
assert isinstance('a', (int, (str, bytes))), 'isinstance nested tuple match'
assert isinstance(1, ((str, float), int)), 'isinstance deeply nested'
assert not isinstance([], (int, (str, bytes))), 'isinstance nested tuple no match'

# NoneType capitalization
assert repr(type(None)) == "<class 'NoneType'>", 'NoneType capitalized'

# === type().__name__ ===
assert type(42).__name__ == 'int', 'int type name'
assert type('hello').__name__ == 'str', 'str type name'
assert type(True).__name__ == 'bool', 'bool type name'
assert type(None).__name__ == 'NoneType', 'NoneType name'
assert type([1, 2]).__name__ == 'list', 'list type name'
assert type({'a': 1}).__name__ == 'dict', 'dict type name'

# type().__name__ for exceptions
try:
    raise ValueError('test')
except ValueError as e:
    assert type(e).__name__ == 'ValueError', 'exception type name'



# This test case is also used in the benchmark (benches/main.rs)
# List operations
my_list = []
my_list.append(1)
my_list.append(2)
my_list.insert(0, 0)
list_len = len(my_list)
list_item = my_list[1]

# Dict operations
my_dict = {}
my_dict['a'] = 10
my_dict['b'] = 20
dict_val = my_dict['a']
popped = my_dict.pop('b')
dict_len = len(my_dict)

# Tuple operations
my_tuple = (1, 2, 3)
tuple_item = my_tuple[0]
tuple_len = len(my_tuple)

# String operations
s = 'hello'
s += ' world'
str_len = len(s)


# Function definition and call
def add(x, y):
    return x + y


func_result = add(3, 4)

# For loop with if/elif/else
total = 0
for i in range(10):
    if i < 3:
        total += 1
    elif i < 6:
        total += 2
    else:
        total += 3

# Boolean operators and comparisons
flag = True and not False
check = 1 < 2 and 3 > 2
identity = None is None
not_identity = 1 is not None
compare = 5 >= 5 and 5 <= 5 and 4 != 5

# Assert with message
assert total > 0, 'total should be positive'

# List comprehension
squares = [x * x for x in range(10)]
comp_sum = sum(squares)

# Dict comprehension
square_dict = {x: x * x for x in range(5)}
dict_comp_sum = sum(square_dict.values())

# Final result
result = list_len + list_item + dict_val + dict_len + tuple_item + tuple_len
result += str_len + func_result + total + comp_sum + dict_comp_sum
result
# Return=373



import queue

# === module attributes ===
assert hasattr(queue, 'Queue'), 'queue.Queue should exist'
assert hasattr(queue, 'LifoQueue'), 'queue.LifoQueue should exist'
assert hasattr(queue, 'PriorityQueue'), 'queue.PriorityQueue should exist'
assert hasattr(queue, 'SimpleQueue'), 'queue.SimpleQueue should exist'
assert hasattr(queue, 'Empty'), 'queue.Empty should exist'
assert hasattr(queue, 'Full'), 'queue.Full should exist'

# === Queue FIFO behavior ===
q = queue.Queue()
assert q.qsize() == 0, 'Queue should start empty'
assert q.empty() is True, 'Queue.empty() should be true initially'
assert q.full() is False, 'Queue.full() should be false for maxsize=0'

q.put(10)
q.put(20)
assert q.qsize() == 2, 'Queue size should increase on put'
assert q.get() == 10, 'Queue.get() should return FIFO first item'
assert q.get_nowait() == 20, 'Queue.get_nowait() should return next FIFO item'
assert q.empty() is True, 'Queue should be empty after removing all items'

# task_done/join should be callable without deadlocking when bookkeeping is balanced
assert q.task_done() is None, 'Queue.task_done() should exist and return None'
assert q.task_done() is None, 'Queue.task_done() should be callable per retrieved item'
assert q.join() is None, 'Queue.join() should return when unfinished task count is zero'

# === Queue(maxsize) full behavior ===
q2 = queue.Queue(1)
q2.put('x')
assert q2.full() is True, 'Queue.full() should report true when maxsize reached'

try:
    q2.put_nowait('y')
    assert False, 'Queue.put_nowait on full queue should raise queue.Full'
except queue.Full:
    pass

# put() accepts block/timeout signature even though Ouros never blocks
try:
    q2.put('y', block=False, timeout=1)
    assert False, 'Queue.put on full queue should raise queue.Full in single-threaded mode'
except queue.Full:
    pass

try:
    q2.put('y', timeout=-1)
    assert False, 'Queue.put timeout<0 should raise ValueError'
except ValueError as exc:
    assert str(exc) == "'timeout' must be a non-negative number", 'timeout validation message should match'

# === Empty behavior ===
q3 = queue.Queue()
try:
    q3.get_nowait()
    assert False, 'Queue.get_nowait on empty queue should raise queue.Empty'
except queue.Empty:
    pass

try:
    q3.get(block=False, timeout=1)
    assert False, 'Queue.get on empty queue should raise queue.Empty'
except queue.Empty:
    pass

try:
    q3.get(timeout=-1)
    assert False, 'Queue.get timeout<0 should raise ValueError'
except ValueError as exc:
    assert str(exc) == "'timeout' must be a non-negative number", 'timeout validation message should match'

# === LifoQueue behavior ===
lifo = queue.LifoQueue()
lifo.put(1)
lifo.put(2)
lifo.put(3)
assert lifo.get() == 3, 'LifoQueue should pop latest item first'
assert lifo.get_nowait() == 2, 'LifoQueue get_nowait should be LIFO'
assert lifo.get() == 1, 'LifoQueue should eventually return oldest item'

# === PriorityQueue behavior ===
pq = queue.PriorityQueue()
pq.put((2, 'b'))
pq.put((1, 'a'))
pq.put((3, 'c'))
assert pq.get() == (1, 'a'), 'PriorityQueue should return smallest item first'
assert pq.get_nowait() == (2, 'b'), 'PriorityQueue should keep min-heap order'
assert pq.get() == (3, 'c'), 'PriorityQueue should return remaining item'

# === SimpleQueue behavior ===
sq = queue.SimpleQueue()
assert sq.empty() is True, 'SimpleQueue should start empty'
assert sq.qsize() == 0, 'SimpleQueue initial size should be 0'

# SimpleQueue.put accepts block/timeout for compatibility but stays unbounded
assert sq.put('a') is None, 'SimpleQueue.put should return None'
assert sq.put('b', block=False, timeout=123) is None, 'SimpleQueue.put should accept block/timeout kwargs'
assert sq.qsize() == 2, 'SimpleQueue size should track puts'
assert sq.get() == 'a', 'SimpleQueue should be FIFO'
assert sq.get_nowait() == 'b', 'SimpleQueue.get_nowait should return next FIFO item'
assert sq.empty() is True, 'SimpleQueue should be empty after gets'

try:
    sq.get_nowait()
    assert False, 'SimpleQueue.get_nowait on empty queue should raise queue.Empty'
except queue.Empty:
    pass

# === from import ===
from queue import LifoQueue, PriorityQueue, Queue, SimpleQueue

assert Queue is not None, 'from queue import Queue should work'
assert LifoQueue is not None, 'from queue import LifoQueue should work'
assert PriorityQueue is not None, 'from queue import PriorityQueue should work'
assert SimpleQueue is not None, 'from queue import SimpleQueue should work'



lst = [1, 2, 3]
lst[10] = 'value'
"""
TRACEBACK:
Traceback (most recent call last):
  File "list__setitem_index_error.py", line 2, in <module>
    lst[10] = 'value'
    ~~~~~~~
IndexError: list assignment index out of range
"""



# call-external
# External calls in augmented assignment expressions

# += with external call
x = 10
x += add_ints(5, 5)
assert x == 20, 'ext call in augmented add'

# -= with external call
x = 100
x -= add_ints(20, 30)
assert x == 50, 'ext call in augmented sub'

# *= with external call
x = 5
x *= add_ints(2, 1)
assert x == 15, 'ext call in augmented mul'



# === Basic pack/unpack with integers ===
import struct

# Test basic integer formats
assert struct.pack('b', 42) == b'*', 'signed byte pack failed'
assert struct.unpack('b', b'*')[0] == 42, 'signed byte unpack failed'

assert struct.pack('B', 200) == b'\xc8', 'unsigned byte pack failed'
assert struct.unpack('B', b'\xc8')[0] == 200, 'unsigned byte unpack failed'

assert struct.pack('h', 1000) == b'\xe8\x03', 'short pack failed (little-endian)'
assert struct.unpack('h', b'\xe8\x03')[0] == 1000, 'short unpack failed'

assert struct.pack('i', 100000) == b'\xa0\x86\x01\x00', 'int pack failed (little-endian)'
assert struct.unpack('i', b'\xa0\x86\x01\x00')[0] == 100000, 'int unpack failed'

# === Byte order markers ===
assert struct.pack('<i', 1) == b'\x01\x00\x00\x00', 'little-endian pack failed'
assert struct.pack('>i', 1) == b'\x00\x00\x00\x01', 'big-endian pack failed'
assert struct.pack('!i', 1) == b'\x00\x00\x00\x01', 'network byte order pack failed'

# Unpack with different byte orders
assert struct.unpack('<i', b'\x01\x00\x00\x00')[0] == 1, 'little-endian unpack failed'
assert struct.unpack('>i', b'\x00\x00\x00\x01')[0] == 1, 'big-endian unpack failed'

# === calcsize ===
assert struct.calcsize('b') == 1, 'calcsize b should be 1'
assert struct.calcsize('h') == 2, 'calcsize h should be 2'
assert struct.calcsize('i') == 4, 'calcsize i should be 4'
assert struct.calcsize('q') == 8, 'calcsize q should be 8'
assert struct.calcsize('f') == 4, 'calcsize f should be 4'
assert struct.calcsize('d') == 8, 'calcsize d should be 8'
assert struct.calcsize('3i') == 12, 'calcsize 3i should be 12'

# Long size is platform dependent (8 on macOS ARM64, 4 on Windows)
# Just check that calcsize returns a positive number
assert struct.calcsize('l') > 0, 'calcsize l should be positive'
assert struct.calcsize('q') == 8, 'calcsize q should be 8'

# === Float formats ===
packed_float = struct.pack('f', 3.14)
assert len(packed_float) == 4, 'float should pack to 4 bytes'
unpacked_float = struct.unpack('f', packed_float)[0]
assert abs(unpacked_float - 3.14) < 0.01, 'float unpack incorrect'

packed_double = struct.pack('d', 3.14159)
assert len(packed_double) == 8, 'double should pack to 8 bytes'
unpacked_double = struct.unpack('d', packed_double)[0]
assert abs(unpacked_double - 3.14159) < 0.0001, 'double unpack incorrect'

# === Bool format ===
assert struct.pack('?', True) == b'\x01', 'bool True pack failed'
assert struct.pack('?', False) == b'\x00', 'bool False pack failed'
assert struct.unpack('?', b'\x01')[0] == True, 'bool True unpack failed'
assert struct.unpack('?', b'\x00')[0] == False, 'bool False unpack failed'

# === String format (bytes) ===
assert struct.pack('5s', b'hello') == b'hello', 'string pack failed'
assert struct.unpack('5s', b'hello')[0] == b'hello', 'string unpack failed'

# Padding with shorter strings
assert struct.pack('5s', b'hi') == b'hi\x00\x00\x00', 'short string pack should pad with nulls'

# === Pad bytes ===
assert struct.pack('x') == b'\x00', 'pad byte pack failed'
assert struct.unpack('x', b'\x00') == (), 'pad byte unpack should return empty tuple'
assert struct.calcsize('x') == 1, 'calcsize x should be 1'
assert struct.calcsize('3x') == 3, 'calcsize 3x should be 3'

# === Multiple values ===
packed = struct.pack('ii', 1, 2)
assert len(packed) == 8, 'two ints should pack to 8 bytes'
assert struct.unpack('ii', packed) == (1, 2), 'two int unpack failed'

# Mixed format
packed = struct.pack('ihb', 1000, 500, 50)
# calcsize ihb is platform dependent (7 on standard, might be 8 on native with alignment)
calc_size = struct.calcsize('ihb')
assert calc_size >= 7, f'calcsize ihb should be at least 7, got {calc_size}'
assert struct.unpack('ihb', packed) == (1000, 500, 50), 'mixed format unpack failed'

# === Long long (64-bit) ===
packed_q = struct.pack('q', 9223372036854775807)  # Max i64
assert len(packed_q) == 8, 'long long should pack to 8 bytes'
assert struct.unpack('q', packed_q)[0] == 9223372036854775807, 'long long unpack failed'

# Note: unsigned long long can hold values from 0 to 18446744073709551615
# but Ouros's current implementation handles i64 range best
packed_Q = struct.pack('Q', 9223372036854775807)  # Max i64
assert len(packed_Q) == 8, 'unsigned long long should pack to 8 bytes'

# === Char format ===
packed_char = struct.pack('c', b'A')
assert packed_char == b'A', 'char pack failed'
# Note: unpack returns a bytes object of length 1, not a string
unpacked_char = struct.unpack('c', b'A')[0]
assert unpacked_char == b'A', 'char unpack failed'

# === Pascal string ===
# Pascal strings have a length byte followed by data
packed_pascal = struct.pack('5p', b'hi')
assert packed_pascal[0] == 2, 'pascal string length byte incorrect'
assert packed_pascal[1:3] == b'hi', 'pascal string data incorrect'

unpacked_pascal = struct.unpack('5p', packed_pascal)[0]
assert unpacked_pascal == b'hi', 'pascal string unpack failed'

# === Whitespace in format ===
assert struct.pack(' i ', 42) == struct.pack('i', 42), 'whitespace in format should be ignored'
assert struct.calcsize(' i ') == 4, 'whitespace should not affect calcsize'

# === Module attributes ===
assert hasattr(struct, 'error'), 'struct should have error attribute'
assert hasattr(struct, 'pack'), 'struct should have pack function'
assert hasattr(struct, 'unpack'), 'struct should have unpack function'
assert hasattr(struct, 'calcsize'), 'struct should have calcsize function'
assert hasattr(struct, 'iter_unpack'), 'struct should have iter_unpack function'
assert hasattr(struct, 'pack_into'), 'struct should have pack_into function'
assert hasattr(struct, 'unpack_from'), 'struct should have unpack_from function'

# === iter_unpack basic test ===
# Pack values - iter_unpack should handle multiple records
packed = struct.pack('ii', 1, 2)
result = list(struct.iter_unpack('ii', packed))
# iter_unpack yields tuples
assert len(result) >= 1, 'iter_unpack should yield at least one tuple'
assert result[0] == (1, 2), 'iter_unpack first tuple should match packed values'

# === unpack_from basic test ===
packed = b'\x00\x00' + struct.pack('i', 42)  # 2 bytes padding, then int
result = struct.unpack_from('i', packed, 2)
assert result[0] == 42, 'unpack_from with offset failed'

# === Negative numbers ===
assert struct.unpack('b', struct.pack('b', -1))[0] == -1, 'negative byte failed'
assert struct.unpack('h', struct.pack('h', -1000))[0] == -1000, 'negative short failed'
assert struct.unpack('i', struct.pack('i', -100000))[0] == -100000, 'negative int failed'

# === Zero values ===
assert struct.pack('i', 0) == b'\x00\x00\x00\x00', 'zero int pack failed'
assert struct.unpack('i', b'\x00\x00\x00\x00')[0] == 0, 'zero int unpack failed'

# === Standard size mode (=) ===
# In standard mode, long is 4 bytes regardless of platform
assert struct.calcsize('=l') == 4, 'calcsize =l should be 4 (standard)'
assert struct.calcsize('=L') == 4, 'calcsize =L should be 4 (standard)'



import base64

# === b64encode ===
assert base64.b64encode(b'') == b'', 'encode empty'
assert base64.b64encode(b'hello') == b'aGVsbG8=', 'encode hello'
assert base64.b64encode(b'Hello, World!') == b'SGVsbG8sIFdvcmxkIQ==', 'encode hello world'
assert base64.b64encode(b'a') == b'YQ==', 'encode single char'
assert base64.b64encode(b'ab') == b'YWI=', 'encode two chars'
assert base64.b64encode(b'abc') == b'YWJj', 'encode three chars'
assert base64.b64encode(b'Python') == b'UHl0aG9u', 'encode Python'

# === b64decode ===
assert base64.b64decode(b'') == b'', 'decode empty'
assert base64.b64decode(b'aGVsbG8=') == b'hello', 'decode hello'
assert base64.b64decode(b'SGVsbG8sIFdvcmxkIQ==') == b'Hello, World!', 'decode hello world'
assert base64.b64decode(b'YQ==') == b'a', 'decode single char'
assert base64.b64decode(b'YWI=') == b'ab', 'decode two chars'
assert base64.b64decode(b'YWJj') == b'abc', 'decode three chars'

# === roundtrip ===
for text in [b'test', b'ouros python', b'12345', b'special chars: @#$%']:
    assert base64.b64decode(base64.b64encode(text)) == text, 'roundtrip: ' + text.decode()

# === from import ===
from base64 import b64decode, b64encode

assert b64encode(b'test') == b'dGVzdA==', 'from import b64encode'
assert b64decode(b'dGVzdA==') == b'test', 'from import b64decode'



# === range() with kwargs error (lines 152-154) ===
try:
    range(stop=5)
except TypeError as e:
    assert 'keyword' in str(e).lower() or 'kwargs' in str(e).lower(), 'range with kwargs raises TypeError'

# === range() with too many args via ArgsKargs (lines 162-163) ===
# This is triggered when >3 positional args are passed
args = [1, 2, 3, 4]
try:
    range(*args)
except TypeError as e:
    assert 'most' in str(e) or '3' in str(e), 'range with 4+ args raises TypeError'

# === range slicing (lines 209, 223-225, 229-231, 233-235) ===
r = range(10)
sliced = r[2:7]
assert list(sliced) == [2, 3, 4, 5, 6], 'range slice forward'

r = range(10)
sliced = r[::2]
assert list(sliced) == [0, 2, 4, 6, 8], 'range slice with step 2'

r = range(10)
sliced = r[::-1]
assert list(sliced) == [9, 8, 7, 6, 5, 4, 3, 2, 1, 0], 'range reverse slice'

r = range(0, 20, 3)
sliced = r[1:4]
assert list(sliced) == [3, 6, 9], 'range with step slice'

# Range slice with negative step
r = range(10)
sliced = r[8:2:-2]
assert list(sliced) == [8, 6, 4], 'range slice with negative step'

# Empty range slice
r = range(10)
sliced = r[5:5]
assert list(sliced) == [], 'range empty slice (equal bounds)'

r = range(10)
sliced = r[7:3]
assert list(sliced) == [], 'range empty slice (start > stop)'

# === range.index() (line 301) ===
# range doesn't have .index() in ouros but let's test range containment paths
# that hit py_dec_ref_ids and py_estimate_size (lines 301, 303, 305-307)

# These lines are for py_dec_ref_ids (noop), py_estimate_size, and end of impl
# They get hit by general usage patterns
r = range(100)
assert len(r) == 100, 'len of large range'
assert repr(r) == 'range(0, 100)', 'repr of range'
assert bool(r) == True, 'non-empty range is truthy'

# === Range iteration exercises Range struct ===
total = 0
for i in range(1, 11, 2):
    total += i
assert total == 25, 'sum of odd range 1-10'

# === Range with negative step and containment ===
r = range(10, -10, -3)
assert list(r) == [10, 7, 4, 1, -2, -5, -8], 'range with negative step values'
assert 10 in r, '10 in negative step range'
assert 7 in r, '7 in negative step range'
assert -8 in r, '-8 in negative step range'
assert 9 not in r, '9 not in negative step range (off step)'
assert -9 not in r, '-9 not in negative step range (off step)'

# === Range equality edge cases ===
# Empty ranges are always equal regardless of parameters
assert range(0) == range(0), 'empty ranges equal'
assert range(5, 5) == range(10, 10), 'different empty ranges equal'
assert range(5, 5, 2) == range(10, 10, 3), 'empty ranges with different steps equal'
assert range(0, 0, -1) == range(0), 'empty backwards range equals empty forward'

# Non-empty ranges
assert range(0, 10, 2) != range(0, 10, 3), 'different steps not equal'
assert range(1, 10, 2) != range(2, 10, 2), 'different starts not equal'

# === Range getitem with slice step 0 error ===
try:
    range(10)[::0]
except ValueError as e:
    assert 'step' in str(e).lower() or 'zero' in str(e).lower(), 'range slice step 0 raises ValueError'

# === Range as iterable in list constructor ===
assert list(range(5, 0, -1)) == [5, 4, 3, 2, 1], 'list from countdown range'

# === Range with large step ===
r = range(0, 100, 50)
assert list(r) == [0, 50], 'range with large step'
assert len(r) == 2, 'len of range with large step'
assert r[0] == 0, 'first element of large step range'
assert r[1] == 50, 'second element of large step range'



# Test that accessing a variable before assignment in a function raises UnboundLocalError
# (In function scope, Python pre-scans for assignments so it knows x is local)
def foo():
    print(x)
    x = 1


foo()
"""
TRACEBACK:
Traceback (most recent call last):
  File "name_error__unbound_local_func.py", line 8, in <module>
    foo()
    ~~~~~
  File "name_error__unbound_local_func.py", line 4, in foo
    print(x)
          ~
UnboundLocalError: cannot access local variable 'x' where it is not associated with a value
"""



# === Generator expression returns a generator, not a list ===
g = (x for x in range(5))
assert type(g).__name__ == 'generator', 'generator expression creates generator object'

# === Generator expression is lazy (not eagerly evaluated) ===
evaluated = []


def track(x):
    evaluated.append(x)
    return x


g = (track(x) for x in range(3))
assert evaluated == [], 'generator expression is lazy, nothing evaluated yet'
next(g)
assert evaluated == [0], 'only first element evaluated after one next()'
next(g)
assert evaluated == [0, 1], 'second element evaluated after two next() calls'
next(g)
assert evaluated == [0, 1, 2], 'third element evaluated after three next() calls'

# === Generator expression consumed by list() ===
result = list(x * 2 for x in range(5))
assert result == [0, 2, 4, 6, 8], 'generator expression consumed by list()'

# === Generator expression in sum() ===
assert sum(x for x in range(5)) == 10, 'generator expression in sum()'

# === Generator expression in any() ===
assert any(x > 3 for x in range(5)), 'generator expression in any()'
assert not any(x > 10 for x in range(5)), 'generator expression in any() false'

# === Generator expression in all() ===
assert all(x < 5 for x in range(5)), 'generator expression in all()'
assert not all(x < 3 for x in range(5)), 'generator expression in all() false'

# === Generator expression with condition ===
result = list(x for x in range(10) if x % 2 == 0)
assert result == [0, 2, 4, 6, 8], 'generator expression with condition'

# === Generator expression with transformation and condition ===
result = list(x * x for x in range(6) if x % 2 != 0)
assert result == [1, 9, 25], 'generator expression with transform and condition'

# === Nested generator expression ===
result = list(x + y for x in range(3) for y in range(2))
assert result == [0, 1, 1, 2, 2, 3], 'nested generator expression'

# === Generator expression is single-use (exhaustion) ===
g = (x for x in range(3))
first = list(g)
second = list(g)
assert first == [0, 1, 2], 'first consumption of generator expression'
assert second == [], 'second consumption of exhausted generator expression'

# === Generator expression __iter__ returns self ===
g = (x for x in range(3))
assert iter(g) is g, 'generator expression __iter__ returns self'

# === Generator expression in for loop ===
result = []
for x in (i * 10 for i in range(4)):
    result.append(x)
assert result == [0, 10, 20, 30], 'generator expression in for loop'

# === Generator expression with tuple unpacking ===
pairs = [(1, 2), (3, 4), (5, 6)]
result = list(a + b for a, b in pairs)
assert result == [3, 7, 11], 'generator expression with tuple unpacking'

# === Generator expression captures variables from enclosing scope ===
multiplier = 3
result = list(x * multiplier for x in range(4))
assert result == [0, 3, 6, 9], 'generator expression captures enclosing scope variable'

# === Generator expression with multiple conditions ===
result = list(x for x in range(20) if x % 2 == 0 if x % 3 == 0)
assert result == [0, 6, 12, 18], 'generator expression with multiple conditions'

# === Generator expression with string operations ===
words = ['hello', 'world', 'foo']
result = list(w + '!' for w in words)
assert result == ['hello!', 'world!', 'foo!'], 'generator expression with strings'

# === Generator expression passed directly to tuple() ===
result = tuple(x * x for x in range(5))
assert result == (0, 1, 4, 9, 16), 'generator expression to tuple'

# === Empty generator expression ===
result = list(x for x in [])
assert result == [], 'empty generator expression'

# === Generator expression where condition filters everything ===
result = list(x for x in range(5) if x > 100)
assert result == [], 'generator expression where condition filters all'

# === Generator expression with bool conversion ===
result = list(not x for x in [True, False, True])
assert result == [False, True, False], 'generator expression with bool conversion'


# === Nested function with generator expression ===
def make_gen(n):
    return (x * 2 for x in range(n))


assert list(make_gen(4)) == [0, 2, 4, 6], 'function returning generator expression'

# === Generator expression: outermost iterator created eagerly, but list mutation is visible ===
# CPython calls iter() on the outermost iterable at creation time.
# For a list, the iterator sees mutations to the list (appends, etc.)
# because list iterators check length dynamically.
source = [1, 2, 3]
g = (x * 10 for x in source)
source.append(4)  # mutation visible because list iterator checks length
result = list(g)
assert result == [10, 20, 30, 40], 'list mutations visible through generator expression iterator'

# Replacing the source entirely after creation does not affect the generator.
# The generator holds a reference to the original list via the iterator.
source2 = [1, 2, 3]
g2 = (x for x in source2)
source2 = [99]  # rebind, does not affect the iterator
assert list(g2) == [1, 2, 3], 'rebinding source variable does not affect generator'



# call-external
# === External function exceptions ===
# Tests for exceptions raised by external functions

# === Basic exception propagation ===

# External function raising ValueError
caught_value_error = False
try:
    result = raise_error('ValueError', 'test error')
    assert False, 'should not reach here'
except ValueError:
    caught_value_error = True
assert caught_value_error, 'ValueError was caught'

# External function raising TypeError
caught_type_error = False
try:
    result = raise_error('TypeError', 'type error message')
    assert False, 'should not reach here'
except TypeError:
    caught_type_error = True
assert caught_type_error, 'TypeError was caught'

# External function raising KeyError
caught_key_error = False
try:
    result = raise_error('KeyError', 'missing key')
    assert False, 'should not reach here'
except KeyError:
    caught_key_error = True
assert caught_key_error, 'KeyError was caught'

# External function raising RuntimeError
caught_runtime_error = False
try:
    result = raise_error('RuntimeError', 'runtime error')
    assert False, 'should not reach here'
except RuntimeError:
    caught_runtime_error = True
assert caught_runtime_error, 'RuntimeError was caught'

# === Exception not caught by wrong handler ===

# ValueError not caught by TypeError handler
caught_outer = False
try:
    try:
        raise_error('ValueError', 'inner error')
    except TypeError:
        assert False, 'TypeError should not catch ValueError'
except ValueError:
    caught_outer = True
assert caught_outer, 'ValueError caught by outer handler'

# === Exception in expression with multiple ext calls ===

# First ext call raises, second should not be called
try:
    x = raise_error('ValueError', 'first') + add_ints(1, 2)
    assert False, 'should not reach here'
except ValueError:
    pass  # Expected

# === External exception in try body with finally ===

finally_ran = False
try:
    raise_error('ValueError', 'in try')
except ValueError:
    pass  # Caught
finally:
    finally_ran = True
assert finally_ran, 'finally ran after external exception caught'

# External exception propagating through finally
outer_caught = False
finally_ran2 = False
try:
    try:
        raise_error('KeyError', 'will propagate')
    except ValueError:
        assert False, 'ValueError should not catch KeyError'
    finally:
        finally_ran2 = True
except KeyError:
    outer_caught = True
assert finally_ran2, 'finally ran before exception propagated'
assert outer_caught, 'exception propagated after finally'

# === Mix of normal returns and exceptions ===

# Normal return, then exception
value1 = add_ints(10, 20)
assert value1 == 30, 'first ext call returned normally'
try:
    raise_error('ValueError', 'after success')
    assert False, 'should not reach here'
except ValueError:
    pass  # Expected

# Exception, then normal return (after catching)
caught_exc = False
try:
    raise_error('TypeError', 'will be caught')
except TypeError:
    caught_exc = True
value2 = add_ints(5, 5)
assert caught_exc, 'exception was caught'
assert value2 == 10, 'ext call after caught exception returned normally'

# === Exception in except handler from external function ===

outer_catch = False
try:
    try:
        raise ValueError('inner')
    except ValueError:
        raise_error('TypeError', 'from handler')
except TypeError:
    outer_catch = True
assert outer_catch, 'exception from handler caught by outer'

# === Exception in else block from external function ===

else_exc_caught = False
try:
    try:
        pass  # No exception
    except:
        assert False, 'should not reach except'
    else:
        raise_error('RuntimeError', 'from else')
except RuntimeError:
    else_exc_caught = True
assert else_exc_caught, 'exception from else block caught'

# === Exception in finally block ===

# Note: exception in finally replaces any pending exception
finally_exc_caught = False
try:
    try:
        pass
    finally:
        raise_error('ValueError', 'from finally')
except ValueError:
    finally_exc_caught = True
assert finally_exc_caught, 'exception from finally caught'

# === Nested try blocks with external exceptions ===

inner_handled = False
outer_handled = False
finally_count = 0
try:
    try:
        raise_error('ValueError', 'inner error')
    except ValueError:
        inner_handled = True
        raise_error('TypeError', 'from inner handler')
    finally:
        finally_count += 1
except TypeError:
    outer_handled = True
finally:
    finally_count += 1

assert inner_handled, 'inner exception was handled'
assert outer_handled, 'exception from inner handler was caught by outer'
assert finally_count == 2, 'both finally blocks ran'



# === Basic arithmetic ===
assert 1 + 1 == 2, 'add ints'
assert 'a' + 'b' == 'ab', 'add strs'

# === Equality tests ===
assert (1 == 1) == True, 'ints equal true'
assert (1 == 2) == False, 'ints equal false'
assert ('a' == 'a') == True, 'str equal'
assert ('a' == 'b') == False, 'str not equal'
assert ([1, 2] == [1, 2]) == True, 'list equal'
assert ((1, 2) == (1, 2)) == True, 'tuple equal'
assert (b'hello' == b'hello') == True, 'bytes equal'

# === Boolean repr/str ===
assert repr(True) == 'True', 'bool true repr'
assert str(True) == 'True', 'bool true str'
assert repr(False) == 'False', 'bool false repr'
assert str(False) == 'False', 'bool false str'

# === None repr/str ===
assert repr(None) == 'None', 'none repr'
assert str(None) == 'None', 'none str'

# === Ellipsis repr/str ===
assert repr(...) == 'Ellipsis', 'ellipsis repr'
assert str(...) == 'Ellipsis', 'ellipsis str'

# === List repr/str ===
assert repr([1, 2]) == '[1, 2]', 'list repr'
assert str([1, 2]) == '[1, 2]', 'list str'

# === Discard expression result ===
a = 1
[1, 2, 3]  # this list is created and discarded
assert a == 1, 'discard list'

# === Shared list append ===
a = [1]
b = a
b.append(2)
assert len(a) == 2, 'shared list append'

# === For loop string append ===
v = ''
for i in range(1000):
    if i % 13 == 0:
        v = v + 'x'
assert len(v) == 77, 'for loop str append assign'

v = ''
for i in range(1000):
    if i % 13 == 0:
        v += 'x'
assert len(v) == 77, 'for loop str append assign op'



# call-external
# === Basic dataclass tests ===

# Get immutable dataclass from external function
point = make_point()

# === repr and str ===
assert repr(point) == 'Point(x=1, y=2)', f'point repr {point=!r}'
assert str(point) == 'Point(x=1, y=2)', 'point str'

# === Boolean truthiness ===
# Dataclasses are always truthy (like Python class instances)
assert bool(point), 'dataclass bool is True'

# === Hash for immutable dataclass ===
# Immutable (frozen) dataclasses are hashable
h1 = hash(point)
assert h1 != 0, 'hash is not zero'

# Hash is consistent - same object hashes to same value
h2 = hash(point)
assert h1 == h2, 'hash is consistent'

# Equal frozen dataclasses hash to same value
point2 = make_point()
assert hash(point) == hash(point2), 'equal dataclasses have equal hash'

# Frozen dataclass can be used as dict key
d = {point: 'first'}
assert d[point] == 'first', 'frozen dataclass as dict key'
assert d[point2] == 'first', 'equal frozen dataclass looks up same value'

# Frozen dataclass can be added to set
s = {point, point2}
assert len(s) == 1, 'equal frozen dataclasses deduplicated in set'

# Different field values produce different hash
alice = make_user('Alice')
bob = make_user('Bob')
assert hash(alice) != hash(bob), 'different field values have different hash'

# === Mutable dataclass ===
mut_point = make_mutable_point()
assert repr(mut_point) == 'MutablePoint(x=1, y=2)', f'mutable point repr {mut_point=!r}'

# === Dataclass with string argument ===
alice = make_user('Alice')
assert repr(alice) == "User(name='Alice', active=True)", f'user repr with string field {alice=!r}'

# === Dataclass in list (using existing variables) ===
points = [point, mut_point, alice]
assert len(points) == 3, 'dataclass list length'

# === Attribute access (get) ===
# Access fields on immutable dataclass
assert point.x == 1, 'point.x is 1'
assert point.y == 2, 'point.y is 2'

# Access fields on mutable dataclass
assert mut_point.x == 1, 'mut_point.x is 1'
assert mut_point.y == 2, 'mut_point.y is 2'

# Access fields on dataclass with string field
assert alice.name == 'Alice', 'alice.name is Alice'
assert alice.active == True, 'alice.active is True'

# === Attribute assignment (set) ===
# Modify mutable dataclass
mut_point.x = 10
assert mut_point.x == 10, 'mut_point.x updated to 10'
mut_point.y = 20
assert mut_point.y == 20, 'mut_point.y updated to 20'
assert repr(mut_point) == 'MutablePoint(x=10, y=20)', f'repr after attribute update {mut_point=!r}'

# === set other attributes
mut_point.z = 30
assert mut_point.z == 30, 'mut_point.z updated to 30'
assert repr(mut_point) == 'MutablePoint(x=10, y=20)', 'repr after attribute update'

# === Nested attribute access (chained get) ===
# Create outer dataclass with inner dataclass as field
outer = make_mutable_point()
inner = make_mutable_point()
inner.x = 100
inner.y = 200
outer.x = inner

# Chained attribute get: outer.x.y
assert outer.x.x == 100, 'outer.x.x is 100'
assert outer.x.y == 200, 'outer.x.y is 200'

# === Nested attribute assignment (chained set) ===
# Modify nested field via chained access
outer.x.x = 999
assert outer.x.x == 999, 'outer.x.x updated to 999'
outer.x.y = 888
assert outer.x.y == 888, 'outer.x.y updated to 888'

# Verify inner was modified (same object)
assert inner.x == 999, 'inner.x also updated to 999'
assert inner.y == 888, 'inner.y also updated to 888'

# === Deeper nesting (3 levels) ===
level1 = make_mutable_point()
level2 = make_mutable_point()
level3 = make_mutable_point()
level3.x = 42
level2.x = level3
level1.x = level2

# 3-level chained get
assert level1.x.x.x == 42, 'level1.x.x.x is 42'

# 3-level chained set
level1.x.x.x = 7
assert level1.x.x.x == 7, 'level1.x.x.x updated to 7'
assert level3.x == 7, 'level3.x also updated to 7'

# === Empty dataclass ===
empty = make_empty()
assert repr(empty) == 'Empty()', 'empty dataclass repr'
assert str(empty) == 'Empty()', 'empty dataclass str'

# === FrozenInstanceError is subclass of AttributeError ===
# Catching AttributeError should also catch FrozenInstanceError
frozen_point = make_point()
caught = False
try:
    frozen_point.x = 10
except AttributeError:
    caught = True
assert caught, 'FrozenInstanceError caught by AttributeError'



# === range() with one argument (stop) ===
assert list(range(0)) == [], 'range(0) is empty'
assert list(range(1)) == [0], 'range(1) is [0]'
assert list(range(5)) == [0, 1, 2, 3, 4], 'range(5) is [0, 1, 2, 3, 4]'
assert list(range(-3)) == [], 'range negative stop is empty'

# === range() with two arguments (start, stop) ===
assert list(range(0, 3)) == [0, 1, 2], 'range(0, 3)'
assert list(range(1, 5)) == [1, 2, 3, 4], 'range(1, 5)'
assert list(range(5, 10)) == [5, 6, 7, 8, 9], 'range(5, 10)'
assert list(range(3, 3)) == [], 'range equal start stop is empty'
assert list(range(5, 3)) == [], 'range start > stop is empty'
assert list(range(-5, -2)) == [-5, -4, -3], 'range negative to negative'
assert list(range(-3, 2)) == [-3, -2, -1, 0, 1], 'range negative to positive'

# === range() with three arguments (start, stop, step) ===
assert list(range(0, 10, 2)) == [0, 2, 4, 6, 8], 'range step 2'
assert list(range(1, 10, 3)) == [1, 4, 7], 'range step 3'
assert list(range(0, 10, 5)) == [0, 5], 'range step 5'
assert list(range(0, 10, 10)) == [0], 'range step equals diff'
assert list(range(0, 10, 20)) == [0], 'range step > diff'

# === range() with negative step ===
assert list(range(10, 0, -1)) == [10, 9, 8, 7, 6, 5, 4, 3, 2, 1], 'range step -1'
assert list(range(10, 0, -2)) == [10, 8, 6, 4, 2], 'range step -2'
assert list(range(5, 0, -1)) == [5, 4, 3, 2, 1], 'range 5 to 0 step -1'
assert list(range(0, 5, -1)) == [], 'range start < stop with negative step is empty'
assert list(range(-1, -5, -1)) == [-1, -2, -3, -4], 'range negative with negative step'

# === tuple(range()) conversions ===
assert tuple(range(3)) == (0, 1, 2), 'tuple(range(3))'
assert tuple(range(1, 4)) == (1, 2, 3), 'tuple(range(1, 4))'
assert tuple(range(0, 6, 2)) == (0, 2, 4), 'tuple(range(0, 6, 2))'

# === range in for loops ===
total = 0
for i in range(5):
    total = total + i
assert total == 10, 'for loop with range(5)'

total2 = 0
for i in range(1, 4):
    total2 = total2 + i
assert total2 == 6, 'for loop with range(1, 4)'

total3 = 0
for i in range(0, 10, 2):
    total3 = total3 + i
assert total3 == 20, 'for loop with range step 2'

# count down
countdown = []
for i in range(3, 0, -1):
    countdown.append(i)
assert countdown == [3, 2, 1], 'for loop countdown'

# === range repr ===
assert repr(range(5)) == 'range(0, 5)', 'repr range one arg'
assert repr(range(1, 5)) == 'range(1, 5)', 'repr range two args'
assert repr(range(1, 5, 2)) == 'range(1, 5, 2)', 'repr range three args'
assert repr(range(0, 10, 1)) == 'range(0, 10)', 'repr range step 1 omitted'
assert repr(range(5, 0, -1)) == 'range(5, 0, -1)', 'repr range negative step'

# === range type ===
assert type(range(5)) == range, 'type of range'
assert type(range(1, 5)) == range, 'type of range two args'
assert type(range(1, 5, 2)) == range, 'type of range three args'

# === range equality ===
assert range(5) == range(5), 'range equality same'
assert range(0, 5) == range(5), 'range(0, 5) == range(5)'
assert range(1, 5) == range(1, 5), 'range equality two args'
assert range(1, 5, 2) == range(1, 5, 2), 'range equality three args'
assert range(5) != range(6), 'range inequality'
assert range(1, 5) != range(2, 5), 'range inequality start differs'
assert range(1, 5, 1) != range(1, 5, 2), 'range inequality step differs'

# === range bool (truthiness) ===
assert bool(range(5)) == True, 'non-empty range is truthy'
assert bool(range(1, 5)) == True, 'range(1, 5) is truthy'
assert bool(range(0)) == False, 'empty range(0) is falsy'
assert bool(range(5, 5)) == False, 'empty range equal start stop is falsy'
assert bool(range(5, 0)) == False, 'empty range start > stop is falsy'
assert bool(range(5, 0, -1)) == True, 'range countdown is truthy'
assert bool(range(0, 5, -1)) == False, 'empty range wrong direction is falsy'

# === range isinstance ===
assert isinstance(range(5), range), 'isinstance range'

# === len(range()) ===
assert len(range(5)) == 5, 'len(range(5))'
assert len(range(0)) == 0, 'len(range(0))'
assert len(range(1, 5)) == 4, 'len(range(1, 5))'
assert len(range(0, 10, 2)) == 5, 'len(range step 2)'
assert len(range(10, 0, -1)) == 10, 'len(range negative step)'
assert len(range(0, 10, 3)) == 4, 'len(range step 3)'

# === range equality by sequence (not parameters) ===
assert range(0, 3, 2) == range(0, 4, 2), 'ranges with same sequence [0,2] are equal'
assert range(0, 5, 2) == range(0, 6, 2), 'range(0,5,2) == range(0,6,2) both [0,2,4]'
assert range(5, 0, -2) == range(5, -1, -2), 'negative step same sequence'
assert range(0) == range(0), 'empty ranges equal'
assert range(5, 5) == range(10, 10), 'different empty ranges equal'
assert range(0, 0) == range(5, 5), 'empty ranges with different params equal'

# === Range indexing (getitem) ===
# Basic indexing for range(stop)
r = range(5)
assert r[0] == 0, 'range(5)[0]'
assert r[1] == 1, 'range(5)[1]'
assert r[4] == 4, 'range(5)[4]'

# Negative indexing
assert r[-1] == 4, 'range(5)[-1]'
assert r[-2] == 3, 'range(5)[-2]'
assert r[-5] == 0, 'range(5)[-5]'

# Range with start
r = range(10, 15)
assert r[0] == 10, 'range(10, 15)[0]'
assert r[1] == 11, 'range(10, 15)[1]'
assert r[4] == 14, 'range(10, 15)[4]'
assert r[-1] == 14, 'range(10, 15)[-1]'
assert r[-5] == 10, 'range(10, 15)[-5]'

# Range with step
r = range(0, 10, 2)
assert r[0] == 0, 'range(0, 10, 2)[0]'
assert r[1] == 2, 'range(0, 10, 2)[1]'
assert r[2] == 4, 'range(0, 10, 2)[2]'
assert r[3] == 6, 'range(0, 10, 2)[3]'
assert r[4] == 8, 'range(0, 10, 2)[4]'
assert r[-1] == 8, 'range(0, 10, 2)[-1]'
assert r[-2] == 6, 'range(0, 10, 2)[-2]'

# Range with step 3
r = range(1, 10, 3)
assert r[0] == 1, 'range(1, 10, 3)[0]'
assert r[1] == 4, 'range(1, 10, 3)[1]'
assert r[2] == 7, 'range(1, 10, 3)[2]'
assert r[-1] == 7, 'range(1, 10, 3)[-1]'

# Range with negative step
r = range(10, 0, -1)
assert r[0] == 10, 'range(10, 0, -1)[0]'
assert r[1] == 9, 'range(10, 0, -1)[1]'
assert r[9] == 1, 'range(10, 0, -1)[9]'
assert r[-1] == 1, 'range(10, 0, -1)[-1]'
assert r[-10] == 10, 'range(10, 0, -1)[-10]'

# Range with negative step and larger step
r = range(10, 0, -2)
assert r[0] == 10, 'range(10, 0, -2)[0]'
assert r[1] == 8, 'range(10, 0, -2)[1]'
assert r[2] == 6, 'range(10, 0, -2)[2]'
assert r[3] == 4, 'range(10, 0, -2)[3]'
assert r[4] == 2, 'range(10, 0, -2)[4]'
assert r[-1] == 2, 'range(10, 0, -2)[-1]'

# Range starting from negative
r = range(-5, 0)
assert r[0] == -5, 'range(-5, 0)[0]'
assert r[2] == -3, 'range(-5, 0)[2]'
assert r[-1] == -1, 'range(-5, 0)[-1]'

# Single element range
r = range(42, 43)
assert r[0] == 42, 'single element range[0]'
assert r[-1] == 42, 'single element range[-1]'

# Variable index
r = range(100)
i = 50
assert r[i] == 50, 'range getitem with variable index'

# Bool indices (True=1, False=0)
r = range(10, 15)
assert r[False] == 10, 'range getitem with False'
assert r[True] == 11, 'range getitem with True'

# === Range containment ('in' operator) ===
# Basic containment
assert 0 in range(5), '0 in range(5)'
assert 4 in range(5), '4 in range(5)'
assert 5 not in range(5), '5 not in range(5)'
assert -1 not in range(5), '-1 not in range(5)'

# Range with start
assert 10 in range(10, 15), '10 in range(10, 15)'
assert 14 in range(10, 15), '14 in range(10, 15)'
assert 15 not in range(10, 15), '15 not in range(10, 15)'
assert 9 not in range(10, 15), '9 not in range(10, 15)'

# Range with step
assert 0 in range(0, 10, 2), '0 in range(0, 10, 2)'
assert 2 in range(0, 10, 2), '2 in range(0, 10, 2)'
assert 8 in range(0, 10, 2), '8 in range(0, 10, 2)'
assert 3 not in range(0, 10, 2), '3 not in range(0, 10, 2)'
assert 10 not in range(0, 10, 2), '10 not in range(0, 10, 2)'

# Range with negative step
assert 10 in range(10, 0, -1), '10 in countdown'
assert 1 in range(10, 0, -1), '1 in countdown'
assert 0 not in range(10, 0, -1), '0 not in countdown'
assert 11 not in range(10, 0, -1), '11 not in countdown'

# Negative step with step > 1
assert 10 in range(10, 0, -2), '10 in range(10, 0, -2)'
assert 8 in range(10, 0, -2), '8 in range(10, 0, -2)'
assert 9 not in range(10, 0, -2), '9 not in range(10, 0, -2)'

# Negative ranges
assert -3 in range(-5, 0), '-3 in range(-5, 0)'
assert 0 not in range(-5, 0), '0 not in range(-5, 0)'

# Empty ranges
assert 5 not in range(0), '5 not in empty range'
assert 0 not in range(5, 5), '0 not in empty equal range'

# Non-int types return False (no TypeError)
assert 'a' not in range(5), 'string not in range'

# Float containment (floats equal to integers are contained)
assert 3.0 in range(5), '3.0 in range(5)'
assert 0.0 in range(5), '0.0 in range(5)'
assert 4.0 in range(5), '4.0 in range(5)'
assert 3.5 not in range(5), '3.5 not in range(5)'
assert 5.0 not in range(5), '5.0 not in range(5)'
assert 2.0 in range(0, 10, 2), '2.0 in even range'
assert 3.0 not in range(0, 10, 2), '3.0 not in even range'
assert -1.0 not in range(5), '-1.0 not in range(5)'

# Bool as container element (True=1, False=0 for comparison)
assert True in range(5), 'True in range(5)'
assert False in range(5), 'False in range(5)'
assert True not in range(0), 'True not in empty range'



# === NamedTuple field access via sys.version_info (lines 77, 96-98, 108-110) ===
import sys

vi = sys.version_info
assert vi.major == 3, 'version_info major field'
assert vi.minor >= 0, 'version_info minor field is non-negative'

# === NamedTuple length (lines 146, 153-155) ===
assert len(vi) == 5, 'version_info has 5 fields'

# === NamedTuple indexing (lines 172, 178) ===
assert vi[0] == 3, 'version_info index 0 is major'
assert vi[0] == vi.major, 'index access matches named access'
assert vi[-1] == vi.serial, 'negative index matches last field'

# === NamedTuple bool (lines 214-216) ===
assert bool(vi) == True, 'non-empty named tuple is truthy'

# === NamedTuple equality (lines 182, 185-191, 193-194) ===
# Named tuples compare by value like regular tuples
assert vi == vi, 'named tuple equal to itself'
major = vi[0]
minor = vi[1]
micro = vi[2]
releaselevel = vi[3]
serial = vi[4]
assert vi == (major, minor, micro, releaselevel, serial), 'named tuple equals regular tuple with same values'

# === NamedTuple repr (lines 218-224, 226, 228-236, 239-240) ===
r = repr(vi)
assert 'sys.version_info' in r, 'repr includes type name'
assert 'major=' in r, 'repr includes major field'
assert 'minor=' in r, 'repr includes minor field'
assert 'micro=' in r, 'repr includes micro field'
assert 'releaselevel=' in r, 'repr includes releaselevel field'
assert 'serial=' in r, 'repr includes serial field'

# === NamedTuple py_dec_ref_ids (lines 204-210) ===
# This is exercised when named tuple with ref items is garbage collected
# version_info contains strings which are refs
r2 = repr(vi)
assert isinstance(r2, str), 'repr returns string for named tuple with refs'

# === NamedTuple getattr for known fields ===
assert type(vi.releaselevel) == str, 'releaselevel is a string'
assert type(vi.serial) == int, 'serial is an int'

# === NamedTuple str ===
s = str(vi)
assert 'sys.version_info' in s, 'str includes type name'

# === NamedTuple getitem with out-of-bounds ===
try:
    vi[10]
except IndexError:
    assert True, 'out of bounds index raises IndexError'

try:
    vi[-10]
except IndexError:
    assert True, 'negative out of bounds raises IndexError'

# === NamedTuple getitem with non-int key ===
try:
    vi['major']
except TypeError:
    assert True, 'string key raises TypeError'

# === NamedTuple iteration via tuple() ===
t = tuple(vi)
assert len(t) == 5, 'tuple from named tuple has 5 elements'
assert t[0] == vi.major, 'tuple element matches named field'

# === NamedTuple in comparisons ===
assert vi != (0, 0, 0, '', 0), 'version_info not equal to zeros tuple'



import pytest
from inline_snapshot import snapshot

import ouros

# === SandboxRuntimeError tests ===


def test_zero_division_error():
    m = ouros.Sandbox('1 / 0')
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    # Check that it's also a SandboxError
    assert isinstance(exc_info.value, ouros.SandboxError)
    # Check the inner exception
    inner = exc_info.value.exception()
    assert isinstance(inner, ZeroDivisionError)


def test_value_error():
    m = ouros.Sandbox("raise ValueError('bad value')")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, ValueError)
    assert str(inner) == snapshot('bad value')


def test_type_error():
    m = ouros.Sandbox("'string' + 1")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, TypeError)


def test_index_error():
    m = ouros.Sandbox('[1, 2, 3][10]')
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, IndexError)


def test_key_error():
    m = ouros.Sandbox("{'a': 1}['b']")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, KeyError)


def test_attribute_error():
    m = ouros.Sandbox("raise AttributeError('no such attr')")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, AttributeError)
    assert str(inner) == snapshot('no such attr')


def test_name_error():
    m = ouros.Sandbox('undefined_variable')
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, NameError)


def test_assertion_error():
    m = ouros.Sandbox('assert False')
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, AssertionError)


def test_assertion_error_with_message():
    m = ouros.Sandbox("assert False, 'custom message'")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, AssertionError)
    assert str(inner) == snapshot('custom message')


def test_runtime_error():
    m = ouros.Sandbox("raise RuntimeError('runtime error')")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, RuntimeError)
    assert str(inner) == snapshot('runtime error')


def test_not_implemented_error():
    m = ouros.Sandbox("raise NotImplementedError('not implemented')")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, NotImplementedError)
    assert str(inner) == snapshot('not implemented')


# === SandboxSyntaxError tests ===


def test_syntax_error_on_init():
    with pytest.raises(ouros.SandboxSyntaxError) as exc_info:
        ouros.Sandbox('def')
    # Check that it's also a SandboxError
    assert isinstance(exc_info.value, ouros.SandboxError)
    # Check the inner exception
    inner = exc_info.value.exception()
    assert isinstance(inner, SyntaxError)


def test_syntax_error_unclosed_paren():
    with pytest.raises(ouros.SandboxSyntaxError) as exc_info:
        ouros.Sandbox('print(1')
    inner = exc_info.value.exception()
    assert isinstance(inner, SyntaxError)


def test_syntax_error_invalid_syntax():
    with pytest.raises(ouros.SandboxSyntaxError) as exc_info:
        ouros.Sandbox('x = = 1')
    inner = exc_info.value.exception()
    assert isinstance(inner, SyntaxError)


# === Catching with base class ===


def test_catch_with_base_class():
    m = ouros.Sandbox('1 / 0')
    with pytest.raises(ouros.SandboxError):
        m.run()


def test_catch_syntax_error_with_base_class():
    with pytest.raises(ouros.SandboxError):
        ouros.Sandbox('def')


# === Exception handling within Sandbox ===


def test_raise_caught_exception():
    code = """
try:
    1 / 0
except ZeroDivisionError as e:
    result = 'caught'
result
"""
    m = ouros.Sandbox(code)
    assert m.run() == snapshot('caught')


def test_exception_in_function():
    code = """
def fail():
    raise ValueError('from function')

fail()
"""
    m = ouros.Sandbox(code)
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    inner = exc_info.value.exception()
    assert isinstance(inner, ValueError)
    assert str(inner) == snapshot('from function')


# === Display and str methods ===


def test_display_traceback():
    m = ouros.Sandbox('1 / 0')
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    display = exc_info.value.display()
    assert 'Traceback (most recent call last):' in display
    assert 'ZeroDivisionError' in display


def test_display_type_msg():
    m = ouros.Sandbox("raise ValueError('test message')")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    display = exc_info.value.display('type-msg')
    assert display == snapshot('ValueError: test message')


def test_runtime_display():
    m = ouros.Sandbox("raise ValueError('test message')")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    assert exc_info.value.display('msg') == snapshot('test message')
    assert exc_info.value.display('type-msg') == snapshot('ValueError: test message')
    assert exc_info.value.display() == snapshot("""\
Traceback (most recent call last):
  File "main.py", line 1, in <module>
    raise ValueError('test message')
ValueError: test message\
""")


def test_str_returns_msg():
    m = ouros.Sandbox("raise ValueError('test message')")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    assert str(exc_info.value) == snapshot('ValueError: test message')


def test_syntax_error_display():
    with pytest.raises(ouros.SandboxSyntaxError) as exc_info:
        ouros.Sandbox('def')
    assert exc_info.value.display() == snapshot('Expected an identifier at byte range 3..3')
    assert exc_info.value.display('type-msg') == snapshot('SyntaxError: Expected an identifier at byte range 3..3')


def test_syntax_error_str():
    with pytest.raises(ouros.SandboxSyntaxError) as exc_info:
        ouros.Sandbox('def')
    # str() returns just the message
    assert 'SyntaxError' not in str(exc_info.value)


# === Traceback tests ===


def test_traceback_frames():
    code = """\
def inner():
    raise ValueError('error')

def outer():
    inner()

outer()
"""
    m = ouros.Sandbox(code)
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    frames = exc_info.value.traceback()
    assert isinstance(frames, list)
    assert len(frames) >= 2  # At least module level, outer(), and inner()

    assert exc_info.value.display() == snapshot("""\
Traceback (most recent call last):
  File "main.py", line 7, in <module>
    outer()
    ~~~~~~~
  File "main.py", line 5, in outer
    inner()
    ~~~~~~~
  File "main.py", line 2, in inner
    raise ValueError('error')
ValueError: error\
""")

    assert [f.dict() for f in frames] == snapshot(
        [
            {
                'filename': 'main.py',
                'line': 7,
                'column': 1,
                'end_line': 7,
                'end_column': 8,
                'function_name': '<module>',
                'source_line': 'outer()',
            },
            {
                'filename': 'main.py',
                'line': 5,
                'column': 5,
                'end_line': 5,
                'end_column': 12,
                'function_name': 'outer',
                'source_line': '    inner()',
            },
            {
                'filename': 'main.py',
                'line': 2,
                'column': 11,
                'end_line': 2,
                'end_column': 30,
                'function_name': 'inner',
                'source_line': "    raise ValueError('error')",
            },
        ]
    )


def test_frame_properties():
    code = """
def foo():
    raise ValueError('test')

foo()
"""
    m = ouros.Sandbox(code)
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    frames = exc_info.value.traceback()

    assert [f.dict() for f in frames] == snapshot(
        [
            {
                'filename': 'main.py',
                'line': 5,
                'column': 1,
                'end_line': 5,
                'end_column': 6,
                'function_name': '<module>',
                'source_line': 'foo()',
            },
            {
                'filename': 'main.py',
                'line': 3,
                'column': 11,
                'end_line': 3,
                'end_column': 29,
                'function_name': 'foo',
                'source_line': "    raise ValueError('test')",
            },
        ]
    )


# === Repr tests ===


def test_runtime_error_repr():
    m = ouros.Sandbox("raise ValueError('test')")
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    assert repr(exc_info.value) == snapshot('SandboxRuntimeError(ValueError: test)')


def test_syntax_error_repr():
    with pytest.raises(ouros.SandboxSyntaxError) as exc_info:
        ouros.Sandbox('def')
    assert repr(exc_info.value) == snapshot('SandboxSyntaxError(Expected an identifier at byte range 3..3)')


def test_frame_repr():
    code = """
def foo():
    raise ValueError('test')

foo()
"""
    m = ouros.Sandbox(code)
    with pytest.raises(ouros.SandboxRuntimeError) as exc_info:
        m.run()
    frames = exc_info.value.traceback()
    frame = frames[0]
    assert repr(frame) == snapshot("Frame(filename='main.py', line=5, column=1, function_name='<module>')")



import asyncio
import os
import sys
from dataclasses import dataclass
from pathlib import Path
from typing import assert_type

# === Type checking helper functions ===


def check_int(x: int) -> None:
    pass


def check_float(x: float) -> None:
    pass


def check_str(x: str) -> None:
    pass


def check_bool(x: bool) -> None:
    pass


def check_bytes(x: bytes) -> None:
    pass


def check_list_int(x: list[int]) -> None:
    pass


def check_list_str(x: list[str]) -> None:
    pass


def check_tuple_int(x: tuple[int, ...]) -> None:
    pass


def check_dict_str_int(x: dict[str, int]) -> None:
    pass


def check_set_int(x: set[int]) -> None:
    pass


def check_frozenset_int(x: frozenset[int]) -> None:
    pass


# === Value getter functions ===


def get_int() -> int:
    return 123


def get_float() -> float:
    return 3.14


def get_str() -> str:
    return 'hello'


def get_list_int() -> list[int]:
    return [1, 2, 3]


def get_list_str() -> list[str]:
    return ['a', 'b', 'c']


def get_object() -> object:
    return object()


def get_dict_str_int() -> dict[str, int]:
    return {'a': 1, 'b': 2}


def get_set_str() -> set[str]:
    return {'a', 'b', 'c'}


def get_frozenset_str() -> frozenset[str]:
    return frozenset({'a', 'b', 'c'})


def get_tuple_str_int() -> tuple[str, int]:
    return ('hello', 42)


def get_bytes() -> bytes:
    return b'hello'


# === Core Types ===

obj = object()
t = type(42)


# === Primitive Types ===

# bool
check_bool(True)
check_bool(False)
check_bool(bool(1))
check_bool(bool(''))

# int
check_int(42)
check_int(int('42'))
check_int(int(3.14))
check_int(int(get_int()))
check_int(int(get_float()))

# float
check_float(3.14)
check_float(float('3.14'))
check_float(float(42))
f = get_float()
assert_type(f, float)


# === String and Bytes Types ===

# str
check_str('hello')
check_str(str(42))
check_str(str(b'hello', 'utf-8'))
check_str(str(get_int()))

# bytes
check_bytes(b'hello')
check_bytes(bytes('hello', 'utf-8'))
check_bytes(bytes(10))
check_bytes(bytes([65, 66, 67]))
check_bytes(bytes(get_int()))
k2 = get_bytes()
assert_type(k2, bytes)


# === Container Types ===

# list
check_list_int([1, 2, 3])
check_list_str(list('abc'))
check_list_int(list(range(10)))
m2 = get_list_int()
assert_type(m2, list[int])
m3 = get_list_str()
assert_type(m3, list[str])

# tuple
check_tuple_int(tuple([1, 2, 3]))
p2 = get_tuple_str_int()
assert_type(p2, tuple[str, int])

# dict
check_dict_str_int({'a': 1, 'b': 2})
check_dict_str_int(dict(a=1, b=2))
d = get_dict_str_int()
assert_type(d, dict[str, int])

# set
check_set_int({1, 2, 3})
check_set_int(set([1, 2, 3]))

# frozenset
check_frozenset_int(frozenset([1, 2, 3]))

# range
w = range(get_int())
assert_type(w, range)

# slice
sl1 = slice(10)
sl2 = slice(0, 10)
sl3 = slice(0, 10, 2)


# === Builtin Functions ===

# abs
check_int(abs(-5))
check_float(abs(-3.14))

# all / any
check_bool(all([True, False]))
check_bool(any([True, False]))
aa = all(get_list_int())
assert_type(aa, bool)

# bin / hex / oct
check_str(bin(42))
check_str(hex(255))
check_str(oct(8))
ac = bin(get_int())
assert_type(ac, str)

# chr / ord
check_str(chr(65))
check_int(ord('A'))
af = chr(get_int())
assert_type(af, str)
ag = ord(get_str())
assert_type(ag, int)

# divmod
dm = divmod(10, 3)

# hash
check_int(hash('hello'))
ai = hash(get_str())
assert_type(ai, int)

# id
check_int(id(object()))
ak = id(get_object())
assert_type(ak, int)

# isinstance
check_bool(isinstance(42, int))
al = isinstance(get_object(), int)
assert_type(al, bool)

# len
check_int(len([1, 2, 3]))
an = len(get_list_int())
assert_type(an, int)

# max / min
check_int(max(1, 2, 3))
check_int(min(1, 2, 3))

# pow
check_int(pow(2, 3))
check_float(pow(2.0, 3.0))

# print
aw = print(get_str())
assert_type(aw, None)

# repr
check_str(repr(42))
ax = repr(get_int())
assert_type(ax, str)

# round
check_int(round(3.7))

# sorted
check_list_int(sorted([3, 1, 2]))

# sum
check_int(sum([1, 2, 3]))
ba = sum(get_list_int())
assert_type(ba, int)

# type
bf = type(get_int())
assert_type(bf, type[int])


# === Iterator Types ===

# enumerate
for i_enum, v_enum in enumerate([1, 2, 3]):
    check_int(i_enum)
    check_int(v_enum)

# reversed
for v_rev in reversed([1, 2, 3]):
    check_int(v_rev)

# zip
for a_zip, b_zip in zip([1, 2], ['a', 'b']):
    check_int(a_zip)
    check_str(b_zip)


# === Literal Types ===

bk = None
assert_type(bk, None)


# === Exception Types ===

e1 = BaseException('error')
e2 = Exception('error')
e3 = SystemExit(1)
e4 = KeyboardInterrupt()
e5 = ArithmeticError('error')
e6 = OverflowError('error')
e7 = ZeroDivisionError('error')
e8 = LookupError('error')
e9 = IndexError('error')
e10 = KeyError('key')
e11 = RuntimeError('error')
e12 = NotImplementedError('error')
e13 = RecursionError('error')
e14 = AttributeError('error')
e15 = AssertionError('error')
e16 = MemoryError('error')
e17 = NameError('error')
e18 = SyntaxError('error')
e19 = TimeoutError('error')
e20 = TypeError('error')
e21 = ValueError('error')
e22 = StopIteration()


# === Exception Inheritance ===


def handle_base(e: BaseException) -> None:
    pass


handle_base(Exception('error'))
handle_base(ValueError('error'))
handle_base(KeyError('key'))
handle_base(ZeroDivisionError('error'))


def handle_exception(e: Exception) -> None:
    pass


handle_exception(ValueError('error'))
handle_exception(TypeError('error'))
handle_exception(RuntimeError('error'))


# === Try/Except ===

try:
    x_try = 1 / 0
except ZeroDivisionError as e_try:
    check_str(str(e_try))

try:
    d_try: dict[str, int] = {}
    v_try = d_try['missing']
except KeyError:
    pass

try:
    lst_try: list[int] = []
    v_lst = lst_try[0]
except IndexError:
    pass


# === Raise ===


def may_fail(x_fail: int) -> int:
    if x_fail < 0:
        raise ValueError('x must be non-negative')
    if x_fail == 0:
        raise ZeroDivisionError('x cannot be zero')
    return 100 // x_fail


def not_implemented() -> None:
    raise NotImplementedError('subclass must implement')


print(sys.version)
print(sys.version_info)
print(None, file=sys.stdout)
print(None, file=sys.stderr)

# === async ===


async def foo(a: int):
    return a * 2


async def bar():
    await foo(1)
    await foo(2)
    await foo(3)


await asyncio.gather(bar())  # pyright: ignore


@dataclass
class Point:
    x: int
    y: float


p = Point(1, 2)
assert_type(p.x, int)
assert_type(p.y, float)
p.x = 3
print(p)

path = Path(__file__)
assert_type(path, Path)
# assert_type(path.name, str)

p2 = path.parent
assert_type(p2, Path)

p3 = path / 'test.txt'
assert_type(p3, Path)
assert p3.name == 'test.txt'

x = os.getenv('foobar')
assert_type(x, str | None)

y = os.getenv('foobar', default=int('123'))
assert_type(y, str | int)

x2 = os.environ.get('foobar')
assert_type(x2, str | None)



# === Bytes length ===
assert len(b'') == 0, 'len empty'
assert len(b'hello') == 5, 'len basic'

# === Bytes repr/str ===
assert repr(b'hello') == "b'hello'", 'bytes repr'
assert str(b'hello') == "b'hello'", 'bytes str'

# === Various bytes repr cases ===
assert repr(b'') == "b''", 'empty bytes repr'
assert repr(b"it's") == 'b"it\'s"', 'single quote bytes repr'
assert repr(b'l1\nl2') == "b'l1\\nl2'", 'newline bytes repr'
assert repr(b'col1\tcol2') == "b'col1\\tcol2'", 'tab bytes repr'
assert repr(b'\x00\xff') == "b'\\x00\\xff'", 'non-printable bytes repr'
assert repr(b'back\\slash') == "b'back\\\\slash'", 'backslash bytes repr'

# === Bytes containment (in / not in) ===
assert 104 in b'hello', 'int in bytes'
assert 120 not in b'hello', 'int not in bytes'
assert b'ell' in b'hello', 'bytes subsequence in bytes'
assert b'' in b'hello', 'empty bytes is contained'

try:
    -1 in b'abc'
    assert False, 'negative int in bytes should raise ValueError'
except ValueError as e:
    assert str(e) == 'byte must be in range(0, 256)', 'bytes contains negative int error'

try:
    'a' in b'abc'
    assert False, 'str in bytes should raise TypeError'
except TypeError as e:
    assert str(e) == "a bytes-like object is required, not 'str'", 'bytes contains str error'

# === Bytes repetition (*) ===
assert b'ab' * 3 == b'ababab', 'bytes mult int'
assert 3 * b'ab' == b'ababab', 'int mult bytes'
assert b'x' * 0 == b'', 'bytes mult zero'
assert b'x' * -1 == b'', 'bytes mult negative'
assert b'' * 5 == b'', 'empty bytes mult'
assert b'ab' * 1 == b'ab', 'bytes mult one'

# === Bytes indexing (getitem) ===
# Basic indexing - returns integer byte values
assert b'hello'[0] == 104, 'bytes getitem index 0 (h=104)'
assert b'hello'[1] == 101, 'bytes getitem index 1 (e=101)'
assert b'hello'[4] == 111, 'bytes getitem last index (o=111)'

# Negative indexing
assert b'hello'[-1] == 111, 'bytes getitem -1 (o=111)'
assert b'hello'[-2] == 108, 'bytes getitem -2 (l=108)'
assert b'hello'[-5] == 104, 'bytes getitem -5 (h=104)'

# Single byte
assert b'x'[0] == 120, 'bytes getitem single byte at 0'
assert b'x'[-1] == 120, 'bytes getitem single byte at -1'

# ASCII printable range
assert b' '[0] == 32, 'bytes getitem space (32)'
assert b'~'[0] == 126, 'bytes getitem tilde (126)'

# Non-printable bytes
assert b'\x00'[0] == 0, 'bytes getitem null byte'
assert b'\xff'[0] == 255, 'bytes getitem 0xff'
assert b'\n'[0] == 10, 'bytes getitem newline'
assert b'\t'[0] == 9, 'bytes getitem tab'

# Heap-allocated bytes
b = bytes(b'abc')
assert b[0] == 97, 'heap bytes getitem 0'
assert b[1] == 98, 'heap bytes getitem 1'
assert b[-1] == 99, 'heap bytes negative getitem'

# Variable index
b = b'xyz'
i = 1
assert b[i] == 121, 'bytes getitem with variable index'

# Verify return type is int
val = b'A'[0]
assert type(val) == int, 'bytes getitem returns int'
assert val == 65, 'bytes getitem value is correct'

# Bool indices (True=1, False=0)
b = b'abc'
assert b[False] == 97, 'bytes getitem with False'
assert b[True] == 98, 'bytes getitem with True'



import linecache


# === Module API ===
for name in ["getline", "getlines", "clearcache", "checkcache", "lazycache", "cache"]:
    assert hasattr(linecache, name), f"missing linecache.{name}"

assert isinstance(linecache.cache, dict), "linecache.cache must be a dict"


# === Missing files / sandbox fallback ===
linecache.clearcache()
assert linecache.cache == {}, "clearcache should empty cache"
assert linecache.getline("definitely_missing.py", 1) == "", "missing line should be empty"
assert linecache.getline("definitely_missing.py", 0) == "", "lineno <= 0 should be empty"
assert linecache.getlines("definitely_missing.py") == [], "missing file should return empty list"
assert linecache.checkcache() is None, "checkcache() should return None"
assert linecache.checkcache("definitely_missing.py") is None, "checkcache(filename) should return None"

lazy_result = linecache.lazycache("definitely_missing.py", {})
assert isinstance(lazy_result, bool), "lazycache should return bool"


# === Cache dictionary behavior ===
linecache.cache["virtual.py"] = (0, None, ["first\n", "second\n"], "virtual.py")
assert linecache.getline("virtual.py", 1) == "first\n", "getline should read cached first line"
assert linecache.getline("virtual.py", 2) == "second\n", "getline should read cached second line"
assert linecache.getline("virtual.py", 3) == "", "getline should return empty when out of range"
assert linecache.getlines("virtual.py") == ["first\n", "second\n"], "getlines should read cached list"


# === Keyword argument handling ===
assert linecache.getline(filename="virtual.py", lineno=2, module_globals=None) == "second\n"
assert linecache.getlines(filename="virtual.py", module_globals=None) == ["first\n", "second\n"]
assert linecache.checkcache(filename="virtual.py") is None
assert isinstance(linecache.lazycache(filename="virtual.py", module_globals={}), bool)


# === Clear cache ===
linecache.clearcache()
assert linecache.cache == {}, "clearcache should clear all cached entries"
assert linecache.getline("virtual.py", 1) == "", "cleared cache should no longer return virtual lines"



# Security: deeply nested and self-referential data structures.
#
# KNOWN GAPS (as of 2026-02-11) — require porting DepthGuard from upstream:
# - Self-referential repr causes Rust RefCell panic ("data already borrowed")
# - Self-referential equality causes stack overflow (SIGABRT)
# - Self-referential tests are commented out until DepthGuard is ported
#
# Active tests below use moderate nesting to verify depth handling works.

# === Nested list creation and access ===
nested = [[[[[[42]]]]]]
assert nested[0][0][0][0][0][0] == 42, 'nested list literal access works'

# === Nested dict creation and access ===
d = {'a': {'b': {'c': {'d': {'e': 99}}}}}
assert d['a']['b']['c']['d']['e'] == 99, 'nested dict literal access works'

# === Build nested list programmatically (moderate depth) ===
deep = [42]
for i in range(20):
    deep = [deep]

val = deep
for i in range(20):
    val = val[0]
assert val == [42], f'programmatic nested list works at depth 20, got {val}'

# === Mixed nesting ===
mixed = {'list': [{'tuple': (1, 2, {'inner': [3, 4]})}]}
assert mixed['list'][0]['tuple'][2]['inner'][1] == 4, 'mixed nesting access works'
r = repr(mixed)
assert isinstance(r, str), 'mixed nested repr returns string'

# === Nested tuple repr ===
t = (1,)
for i in range(10):
    t = (t,)
r = repr(t)
assert isinstance(r, str), 'nested tuple repr returns string'
assert '1' in r, 'nested tuple repr contains inner value'

# === Deep dict repr ===
d2 = {'val': 1}
for i in range(10):
    d2 = {'child': d2}
r = repr(d2)
assert isinstance(r, str), 'nested dict repr returns string'
assert '1' in r, 'nested dict repr contains leaf value'

# ============================================================
# SELF-REFERENTIAL TESTS
# Uncomment after DepthGuard is ported from upstream Ouros.
# These currently crash the host process (RefCell panic + stack overflow).
# ============================================================
#
# a = []
# a.append(a)
# r = repr(a)
# assert r == '[[...]]', 'self-referential list repr'
#
# d = {}
# d['self'] = d
# r = repr(d)
# assert r == "{'self': {...}}", 'self-referential dict repr'
#
# x = []
# x.append(x)
# y = []
# y.append(y)
# try:
#     x == y
# except RecursionError:
#     pass



# call-external
def wrapper():
    raise_error('ValueError', 'from external')


wrapper()
"""
TRACEBACK:
Traceback (most recent call last):
  File "ext_call__exc_in_function.py", line 6, in <module>
    wrapper()
    ~~~~~~~~~
  File "ext_call__exc_in_function.py", line 3, in wrapper
    raise_error('ValueError', 'from external')
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
ValueError: from external
"""



# Tests for typing module stdlib functions and markers.

import typing

# === typing.cast ===
x = typing.cast(int, 'hello')
assert x == 'hello', 'cast should return the value unchanged'


# === typing.get_type_hints ===
def my_func():
    return 42


hints = typing.get_type_hints(my_func)
assert isinstance(hints, dict), 'get_type_hints should return a dict'

# === typing.get_origin ===
origin = typing.get_origin(int)
assert origin is None, 'get_origin on a plain type should return None'

# === typing.get_args ===
args = typing.get_args(int)
assert isinstance(args, tuple), 'get_args should return a tuple'
assert len(args) == 0, 'get_args on a plain type should return empty tuple'

# === Identity-style runtime stubs ===
sample_obj = ['a', 'b']
assert typing.assert_type(sample_obj, typing.Sequence) is sample_obj, 'assert_type should return the same object'
assert typing.reveal_type(sample_obj) is sample_obj, 'reveal_type should return the same object'

# === Decorator/runtime helper stubs ===
overload_result = typing.overload(my_func)
assert overload_result is not None, 'overload should return a value'

try:
    runtime_checkable_result = typing.runtime_checkable(my_func)
except TypeError:

    class MyProtocol(typing.Protocol):
        pass

    runtime_checkable_result = typing.runtime_checkable(MyProtocol)

assert runtime_checkable_result is not None, 'runtime_checkable should return a value'

try:
    dataclass_transformed = typing.dataclass_transform(my_func)
except TypeError:
    dataclass_transformed = typing.dataclass_transform()(my_func)
assert dataclass_transformed is my_func, 'dataclass_transform should behave as a no-op decorator at runtime'

assert typing.override(my_func) is my_func, 'override should be identity'
if hasattr(typing, 'deprecated'):
    assert typing.deprecated(my_func) is my_func, 'deprecated should be identity'

# === typing.NewType ===
result = typing.NewType('MyInt', int)
assert result is not None, 'NewType should return a value'

# === Marker availability (expanded typing compatibility surface) ===
marker_names = [
    'AnyStr',
    'Awaitable',
    'Coroutine',
    'AsyncIterator',
    'AsyncIterable',
    'AsyncGenerator',
    'MutableMapping',
    'MutableSequence',
    'MutableSet',
    'Mapping',
    'Sequence',
    'Set',
    'FrozenSet',
    'DefaultDict',
    'OrderedDict',
    'Counter',
    'Deque',
    'ChainMap',
    'Pattern',
    'Match',
    'IO',
    'TextIO',
    'BinaryIO',
    'NamedTuple',
    'TypedDict',
    'Never',
    'NoReturn',
    'Self',
    'TypeGuard',
    'TypeIs',
    'Unpack',
    'ParamSpec',
    'ParamSpecArgs',
    'ParamSpecKwargs',
    'Concatenate',
    'TypeVarTuple',
    'TypeAlias',
    'Required',
    'NotRequired',
    'SupportsInt',
    'SupportsFloat',
    'SupportsComplex',
    'SupportsBytes',
    'SupportsAbs',
    'SupportsRound',
]

for marker_name in marker_names:
    marker = getattr(typing, marker_name)
    assert marker is not None, f'{marker_name} should exist in typing module'


# === Marker repr checks for names with tricky casing ===
def assert_marker_repr(value, marker_name):
    actual = repr(value)
    assert actual in (
        f'typing.{marker_name}',
        f"<class 'typing.{marker_name}'>",
    ), f'{marker_name} repr mismatch: {actual}'


assert_marker_repr(typing.DefaultDict, 'DefaultDict')
assert_marker_repr(typing.Deque, 'Deque')
assert_marker_repr(typing.Pattern, 'Pattern')
assert_marker_repr(typing.Match, 'Match')
assert_marker_repr(typing.IO, 'IO')
assert_marker_repr(typing.TextIO, 'TextIO')
assert_marker_repr(typing.BinaryIO, 'BinaryIO')
assert_marker_repr(typing.ParamSpecKwargs, 'ParamSpecKwargs')
assert_marker_repr(typing.SupportsRound, 'SupportsRound')


# === Protocol + runtime_checkable ===
class NeedsRun(typing.Protocol):
    def run(self):
        pass


class Runner:
    def run(self):
        return 'ok'


class NoRunner:
    pass


raised_without_runtime_checkable = False
pre_runtime_result = None
try:
    pre_runtime_result = isinstance(Runner(), NeedsRun)
except TypeError:
    raised_without_runtime_checkable = True
if not raised_without_runtime_checkable:
    assert pre_runtime_result == False, 'non-runtime-checkable protocol should not match'


@typing.runtime_checkable
class RuntimeNeedsRun(typing.Protocol):
    def run(self):
        pass


runtime_runner_check = isinstance(Runner(), RuntimeNeedsRun)
runtime_no_runner_check = isinstance(NoRunner(), RuntimeNeedsRun)
assert type(runtime_runner_check) is bool, 'runtime-checkable protocol check should return bool'
assert runtime_no_runner_check == False, 'runtime-checkable protocol should fail missing members'

raised_runtime_non_protocol = False
try:
    typing.runtime_checkable(Runner)
except TypeError:
    raised_runtime_non_protocol = True
assert raised_runtime_non_protocol, 'runtime_checkable should reject non-protocol classes'


# === TypedDict class syntax ===
class Movie(typing.TypedDict):
    name: str
    year: int


movie = Movie(name='Blade Runner', year=1982)
assert isinstance(movie, dict), 'TypedDict class call should return dict'
assert movie == {'name': 'Blade Runner', 'year': 1982}, 'TypedDict class call should map kwargs into dict'
assert Movie.__total__ == True, 'TypedDict class syntax should default total=True'


class PartialMovie(typing.TypedDict, total=False):
    name: str
    year: int


partial_movie = PartialMovie(name='Alien')
assert partial_movie == {'name': 'Alien'}, 'TypedDict total=False should allow partial dict instances'
assert PartialMovie.__total__ == False, 'TypedDict class syntax should respect total=False'

# === TypedDict functional syntax ===
typed_dict_factory = typing.TypedDict
Book = typed_dict_factory('Book', {'title': str, 'pages': int}, total=False)
book = Book(title='Dune')
assert isinstance(book, dict), 'TypedDict functional call should return dict instances'
assert book == {'title': 'Dune'}, 'TypedDict functional instances should accept kwargs mapping'
assert Book.__total__ == False, 'TypedDict functional syntax should propagate total=False'



# === time.time() returns float ===
import time
t1 = time.time()
assert isinstance(t1, float), 'time.time() should return a float'
assert t1 >= 0, 'time.time() should return non-negative value'

# === time.time() is monotonic ===
t2 = time.time()
assert t2 >= t1, 'time.time() should be monotonic'
t3 = time.time()
assert t3 >= t2, 'time.time() should be monotonic across calls'

# === time.monotonic() returns float ===
m1 = time.monotonic()
assert isinstance(m1, float), 'time.monotonic() should return a float'
assert m1 >= 0, 'time.monotonic() should return non-negative value'

# === time.monotonic() is monotonic ===
m2 = time.monotonic()
assert m2 >= m1, 'time.monotonic() should be monotonic'

# === time.monotonic_ns() returns int ===
mn1 = time.monotonic_ns()
assert isinstance(mn1, int), 'time.monotonic_ns() should return an int'
assert mn1 >= 0, 'time.monotonic_ns() should return non-negative value'

# === time.monotonic_ns() is monotonic ===
mn2 = time.monotonic_ns()
assert mn2 >= mn1, 'time.monotonic_ns() should be monotonic'



# === Empty tuple identity (singleton optimization) ===
# In Python, () is () is always True because empty tuples are interned
assert () is (), 'empty tuple identity'
assert tuple() is (), 'tuple() is empty tuple'
assert tuple() is tuple(), 'tuple() identity'
a = ()
b = ()
assert a is b, 'empty tuple vars are same object'
# Empty tuple from operations
assert (1,)[1:] is (), 'slice to empty is singleton'
assert (1, 2) * 0 is (), 'mult by 0 is empty singleton'

# === Tuple length ===
assert len(()) == 0, 'len empty'
assert len((1,)) == 1, 'len single'
assert len((1, 2, 3)) == 3, 'len basic'

# === Tuple indexing ===
a = (1, 2, 3)
assert a[1] == 2, 'getitem basic'

a = ('a', 'b', 'c')
assert a[0 - 2] == 'b', 'getitem negative'
assert a[-1] == 'c', 'getitem -1'

# === Nested tuples ===
assert ((1, 2), (3, 4)) == ((1, 2), (3, 4)), 'nested tuple'

# === Tuple ordering comparisons ===
assert (1, 2) < (1, 3), 'tuple less-than compares element-by-element'
assert (1, 2, 3) > (1, 2, 0), 'tuple greater-than compares first differing element'
assert (1, 2) <= (1, 2), 'tuple less-equal supports equal tuples'
assert (1, 2, 3) >= (1, 2), 'tuple greater-equal handles longer-equal-prefix tuple'

# === Tuple repr/str ===
assert repr((1, 2)) == '(1, 2)', 'tuple repr'
assert str((1, 2)) == '(1, 2)', 'tuple str'

# === Tuple concatenation (+) ===
assert (1, 2) + (3, 4) == (1, 2, 3, 4), 'tuple add basic'
assert () + (1, 2) == (1, 2), 'empty add tuple'
assert (1, 2) + () == (1, 2), 'tuple add empty'
assert () + () == (), 'empty add empty'
assert ('a', 'b') + ('c',) == ('a', 'b', 'c'), 'tuple add strings'
assert ((1, 2),) + ((3, 4),) == ((1, 2), (3, 4)), 'tuple add nested'

# === Tuple repetition (*) ===
assert (1, 2) * 3 == (1, 2, 1, 2, 1, 2), 'tuple mult int'
assert 3 * (1, 2) == (1, 2, 1, 2, 1, 2), 'int mult tuple'
assert (1,) * 0 == (), 'tuple mult zero'
assert (1,) * -1 == (), 'tuple mult negative'
assert () * 5 == (), 'empty tuple mult'
assert (1, 2) * 1 == (1, 2), 'tuple mult one'

# === tuple() constructor ===
assert tuple() == (), 'tuple() empty'
assert tuple([1, 2, 3]) == (1, 2, 3), 'tuple from list'
assert tuple((1, 2, 3)) == (1, 2, 3), 'tuple from tuple'
assert tuple(range(3)) == (0, 1, 2), 'tuple from range'
assert tuple('abc') == ('a', 'b', 'c'), 'tuple from string'
assert tuple(b'abc') == (97, 98, 99), 'tuple from bytes'
assert tuple({'a': 1, 'b': 2}) == ('a', 'b'), 'tuple from dict yields keys'

# non-ASCII strings (multi-byte UTF-8)
assert tuple('héllo') == ('h', 'é', 'l', 'l', 'o'), 'tuple from string with accented char'
assert tuple('日本') == ('日', '本'), 'tuple from string with CJK chars'
assert tuple('a🎉b') == ('a', '🎉', 'b'), 'tuple from string with emoji'



def outer():
    def inner():
        nonlocal x
        return x

    inner()
    x = 1


outer()
"""
TRACEBACK:
Traceback (most recent call last):
  File "traceback__nonlocal_unbound.py", line 10, in <module>
    outer()
    ~~~~~~~
  File "traceback__nonlocal_unbound.py", line 6, in outer
    inner()
    ~~~~~~~
  File "traceback__nonlocal_unbound.py", line 4, in inner
    return x
           ~
NameError: cannot access free variable 'x' where it is not associated with a value in enclosing scope
"""



# call-external
# External calls in return statements


def direct_return():
    return add_ints(10, 20)


result = direct_return()
assert result == 30, 'ext call as direct return value'


def return_with_expression():
    return add_ints(1, 2) + add_ints(3, 4)


result = return_with_expression()
assert result == 10, 'ext call expression as return value'


def conditional_return():
    if return_value(True):
        return add_ints(100, 200)
    return add_ints(1, 1)


result = conditional_return()
assert result == 300, 'ext call in conditional return'



# === Continue in try/finally must run finally ===
result = []
for x in [1, 2, 3]:
    try:
        result.append(x)
        if x == 2:
            continue
        result.append('after-continue')
    finally:
        result.append('finally')
assert result == [1, 'after-continue', 'finally', 2, 'finally', 3, 'after-continue', 'finally'], (
    f'continue in try/finally should run finally: {result}'
)

# === Continue in nested try/finally runs both finally blocks ===
result = []
for x in [1, 2]:
    try:
        try:
            result.append(x)
            continue
        finally:
            result.append('inner-finally')
    finally:
        result.append('outer-finally')
assert result == [1, 'inner-finally', 'outer-finally', 2, 'inner-finally', 'outer-finally'], (
    f'nested finally with continue: {result}'
)

# === Continue in try/except/finally runs finally ===
result = []
for x in [1, 2, 3]:
    try:
        result.append(x)
        if x == 2:
            continue
    except ValueError:
        result.append('except')
    finally:
        result.append('finally')
assert result == [1, 'finally', 2, 'finally', 3, 'finally'], f'continue in try/except/finally: {result}'

# === Continue inside except handler with finally ===
result = []
for x in [1, 2, 3]:
    try:
        if x == 2:
            raise ValueError('test')
        result.append(x)
    except ValueError:
        result.append('except')
        continue
    finally:
        result.append('finally')
    result.append('after')
assert result == [1, 'finally', 'after', 'except', 'finally', 3, 'finally', 'after'], (
    f'continue in except with finally: {result}'
)

# === Continue does not run finally if not in try ===
result = []
for x in [1, 2, 3]:
    result.append(x)
    continue
    result.append('unreachable')
assert result == [1, 2, 3], f'continue without finally: {result}'

# === Continue with multiple loops and finally ===
result = []
for i in [1, 2]:
    try:
        for j in [10, 20, 30]:
            if j == 20:
                continue  # This continue should not trigger outer finally
            result.append(j)
        result.append('after-inner')
    finally:
        result.append('outer-finally')
assert result == [10, 30, 'after-inner', 'outer-finally', 10, 30, 'after-inner', 'outer-finally'], (
    f'inner continue with outer finally: {result}'
)



# Test cycle detection in repr for self-referential structures

# Section 1: List self-reference
a = []
a.append(a)
assert repr(a) == '[[...]]', 'list self-reference'

# Section 2: Dict self-reference
d = {}
d['self'] = d
assert repr(d) == "{'self': {...}}", 'dict self-reference'

# Section 3: Composite - list containing dict containing original list
c = []
e = {'list': c}
c.append(e)
assert repr(c) == "[{'list': [...]}]", 'list containing dict cycle'
assert repr(e) == "{'list': [{...}]}", 'dict containing list cycle'

# Section 4: Multiple references to same cyclic object
f = []
f.append(f)
g = [f, f]
assert repr(g) == '[[[...]], [[...]]]', 'multiple refs to cyclic list'



# call-external
def level4():
    x = 1
    raise_error('RuntimeError', 'deep error')


def level3():
    level4()


def level2():
    level3()


def level1():
    level2()


level1()
"""
TRACEBACK:
Traceback (most recent call last):
  File "ext_call__exc_deep_stack.py", line 19, in <module>
    level1()
    ~~~~~~~~
  File "ext_call__exc_deep_stack.py", line 16, in level1
    level2()
    ~~~~~~~~
  File "ext_call__exc_deep_stack.py", line 12, in level2
    level3()
    ~~~~~~~~
  File "ext_call__exc_deep_stack.py", line 8, in level3
    level4()
    ~~~~~~~~
  File "ext_call__exc_deep_stack.py", line 4, in level4
    raise_error('RuntimeError', 'deep error')
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
RuntimeError: deep error
"""



# === super().__init__ reaches object for explicit object subclasses ===
class ObjectLeaf(object):
    def __init__(self):
        super().__init__()


leaf = ObjectLeaf()
assert isinstance(leaf, ObjectLeaf), 'super().__init__() should resolve object.__init__ for object subclasses'


# === cooperative super() chain ending at object ===
class ChainA:
    def __init__(self):
        self.chain = ['A']
        super().__init__()


class ChainB(ChainA):
    def __init__(self):
        super().__init__()
        self.chain.append('B')


chain = ChainB()
assert chain.chain == ['A', 'B'], 'cooperative super() chain should execute and terminate at object.__init__'


# === diamond super() chain ending at object ===
class DiamondBase:
    def __init__(self, log):
        log.append('base')
        super().__init__()


class DiamondLeft(DiamondBase):
    def __init__(self, log):
        log.append('left')
        super().__init__(log)


class DiamondRight(DiamondBase):
    def __init__(self, log):
        log.append('right')
        super().__init__(log)


class DiamondBottom(DiamondLeft, DiamondRight):
    def __init__(self, log):
        log.append('bottom')
        super().__init__(log)


mro_log = []
DiamondBottom(mro_log)
assert mro_log == ['bottom', 'left', 'right', 'base'], 'diamond super() chain should resolve all __init__ calls'


# === super().__init__ on builtin container subclasses ===
class MyList(list):
    def __init__(self, *args):
        super().__init__(*args)
        self.tag = 'list-ok'


class MyDict(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.tag = 'dict-ok'


class MySet(set):
    def __init__(self, *args):
        super().__init__(*args)
        self.tag = 'set-ok'


my_list = MyList([1, 2])
my_dict = MyDict(a=1)
my_set = MySet([1, 2])

assert my_list.tag == 'list-ok', 'list subclass super().__init__ should execute without AttributeError'
assert my_dict.tag == 'dict-ok', 'dict subclass super().__init__ should execute without AttributeError'
assert my_set.tag == 'set-ok', 'set subclass super().__init__ should execute without AttributeError'


# === exception subclasses still resolve super().__init__ ===
class MyExc(Exception):
    def __init__(self, message):
        super().__init__(message)


try:
    raise MyExc('boom')
except MyExc as exc:
    assert str(exc) == 'boom', 'exception super().__init__ should preserve message'



# xfail=cpython
# time.sleep() raises RuntimeError in sandboxed environment
import time
time.sleep(0.1)
"""
TRACEBACK:
Traceback (most recent call last):
  File "time__sleep_raises.py", line 4, in <module>
    time.sleep(0.1)
    ~~~~~~~~~~~~~~~
RuntimeError: time.sleep() is not supported in sandboxed environment
"""



import token


# === constants ===
assert token.ENDMARKER == 0, 'ENDMARKER'
assert token.NAME == 1, 'NAME'
assert token.NUMBER == 2, 'NUMBER'
assert token.STRING == 3, 'STRING'
assert token.NEWLINE == 4, 'NEWLINE'
assert token.INDENT == 5, 'INDENT'
assert token.DEDENT == 6, 'DEDENT'
assert token.EXCLAMATION == 54, 'EXCLAMATION'
assert token.OP == 55, 'OP'
assert token.TYPE_IGNORE == 56, 'TYPE_IGNORE'
assert token.TYPE_COMMENT == 57, 'TYPE_COMMENT'
assert token.SOFT_KEYWORD == 58, 'SOFT_KEYWORD'
assert token.FSTRING_START == 59, 'FSTRING_START'
assert token.FSTRING_MIDDLE == 60, 'FSTRING_MIDDLE'
assert token.FSTRING_END == 61, 'FSTRING_END'
assert token.TSTRING_START == 62, 'TSTRING_START'
assert token.TSTRING_MIDDLE == 63, 'TSTRING_MIDDLE'
assert token.TSTRING_END == 64, 'TSTRING_END'
assert token.COMMENT == 65, 'COMMENT'
assert token.NL == 66, 'NL'
assert token.ERRORTOKEN == 67, 'ERRORTOKEN'
assert token.ENCODING == 68, 'ENCODING'
assert token.N_TOKENS == 69, 'N_TOKENS'
assert token.NT_OFFSET == 256, 'NT_OFFSET'


# === token names ===
assert token.tok_name[0] == 'ENDMARKER', 'tok_name_endmarker'
assert token.tok_name[token.NAME] == 'NAME', 'tok_name_name'
assert token.tok_name[token.COLONEQUAL] == 'COLONEQUAL', 'tok_name_colonequal'


# === exact token types ===
assert token.EXACT_TOKEN_TYPES['('] == token.LPAR, 'exact_lpar'
assert token.EXACT_TOKEN_TYPES[')'] == token.RPAR, 'exact_rpar'
assert token.EXACT_TOKEN_TYPES['//'] == token.DOUBLESLASH, 'exact_doubleslash'
assert token.EXACT_TOKEN_TYPES[':='] == token.COLONEQUAL, 'exact_walrus'
assert token.EXACT_TOKEN_TYPES['...'] == token.ELLIPSIS, 'exact_ellipsis'
assert token.EXACT_TOKEN_TYPES['!'] == token.EXCLAMATION, 'exact_exclamation'


# === helper predicates ===
assert token.ISTERMINAL(0) == True, 'isterminal_zero'
assert token.ISTERMINAL(1) == True, 'isterminal_name'
assert token.ISTERMINAL(-1) == True, 'isterminal_negative'
assert token.ISTERMINAL(x=0) == True, 'isterminal_keyword'

assert token.ISNONTERMINAL(0) == False, 'isnonterminal_zero'
assert token.ISNONTERMINAL(256) == True, 'isnonterminal_offset'
assert token.ISNONTERMINAL(-1) == False, 'isnonterminal_negative'
assert token.ISNONTERMINAL(x=0) == False, 'isnonterminal_keyword'

assert token.ISEOF(0) == True, 'iseof_zero'
assert token.ISEOF(1) == False, 'iseof_name'
assert token.ISEOF(-1) == False, 'iseof_negative'
assert token.ISEOF(x=0) == True, 'iseof_keyword'



import enum

# === basic import ===
assert enum.Enum is not None, 'Enum is not None'
assert enum.IntEnum is not None, 'IntEnum is not None'
assert str(enum.CONFORM) == 'conform', 'CONFORM constant'
assert str(enum.EJECT) == 'eject', 'EJECT constant'
assert str(enum.KEEP) == 'keep', 'KEEP constant'
assert str(enum.STRICT) == 'strict', 'STRICT constant'

# === from import ===
from enum import CONFORM, EJECT, KEEP, STRICT, Enum, IntEnum

assert Enum is not None, 'from import Enum'
assert IntEnum is not None, 'from import IntEnum'
assert str(CONFORM) == 'conform', 'from import CONFORM'
assert str(EJECT) == 'eject', 'from import EJECT'
assert str(KEEP) == 'keep', 'from import KEEP'
assert str(STRICT) == 'strict', 'from import STRICT'



# === BytesIO Basic Tests ===
from io import BytesIO

b = BytesIO()
assert b.getvalue() == b'', 'Empty BytesIO should have empty value'
assert b.tell() == 0, 'New BytesIO position should be 0'
assert not b.closed, 'New BytesIO should not be closed'
b.close()

b = BytesIO(b'hello')
assert b.getvalue() == b'hello', 'BytesIO with initial value'
b.close()

b = BytesIO(b'')
assert b.getvalue() == b'', 'BytesIO with empty initial value'
b.close()

# === BytesIO Read Tests ===
b = BytesIO(b'hello world')
assert b.read(2) == b'he', 'read(2) should return first 2 bytes'
assert b.read(3) == b'llo', 'read(3) should return next 3 bytes'
assert b.read() == b' world', 'read() should return rest'
assert b.read() == b'', 'read() at EOF should return empty'
b.close()

# Test read1 (should behave like read for BytesIO)
b = BytesIO(b'hello')
assert b.read1(3) == b'hel', 'read1 should work like read'
b.close()

# === BytesIO Write Tests ===
b = BytesIO()
assert b.write(b'hello') == 5, 'write should return byte count'
assert b.getvalue() == b'hello', 'getvalue after write'
b.close()

# Test write at position
b = BytesIO(b'hello world')
b.seek(6)
b.write(b'Python')
assert b.getvalue() == b'hello Python', 'write should overwrite at position'
b.close()

# === BytesIO Seek Tests ===
b = BytesIO(b'hello')
assert b.tell() == 0, 'initial position is 0'
b.seek(3)
assert b.tell() == 3, 'seek(3) should set position to 3'
assert b.read() == b'lo', 'read after seek should work'
b.close()

# Test seek with whence
b = BytesIO(b'hello world')
b.seek(6)
b.seek(3, 1)
assert b.tell() == 9, 'seek from current'
b.seek(-3, 2)
assert b.tell() == 8, 'seek from end'
b.close()

# === BytesIO GetBuffer Test ===
b = BytesIO(b'hello')
buf = b.getbuffer()
assert buf == b'hello', 'getbuffer should return buffer content'
del buf
b.close()

# === BytesIO Close Tests ===
b = BytesIO(b'test')
b.close()
assert b.closed, 'closed should be True after close'

try:
    b.read()
    assert False, 'read on closed should raise'
except ValueError:
    pass

# === BytesIO Boolean Tests ===
b = BytesIO()
assert b.readable(), 'BytesIO should be readable'
assert b.writable(), 'BytesIO should be writable'
assert b.seekable(), 'BytesIO should be seekable'
assert not b.isatty(), 'BytesIO should not be a tty'
b.close()

# === BytesIO ReadLine Tests ===
b = BytesIO(b'line1\nline2\nline3')
assert b.readline() == b'line1\n', 'readline should include newline'
assert b.readline() == b'line2\n', 'readline second call'
assert b.readline() == b'line3', 'readline last without newline'
assert b.readline() == b'', 'readline at EOF'
b.close()

# === BytesIO WriteLines Tests ===
b = BytesIO()
b.writelines([b'a', b'b', b'c'])
assert b.getvalue() == b'abc', 'writelines should concatenate'
b.close()

# === BytesIO Truncate Tests ===
b = BytesIO(b'hello world')
b.truncate(5)
assert b.getvalue() == b'hello', 'truncate should shorten buffer'
b.close()

# === BytesIO Flush Test ===
b = BytesIO()
assert b.flush() is None, 'flush should return None'
b.close()

# === BytesIO Iteration Tests ===
b = BytesIO(b'line1\nline2')
lines = list(b)
assert lines == [b'line1\n', b'line2'], 'iteration should yield lines'
b.close()

# === BytesIO Context Manager Tests ===
with BytesIO() as b:
    b.write(b'test')
    assert b.getvalue() == b'test', 'context manager should work'
assert b.closed, 'stream should be closed after context'



# === String concatenation (+) ===
assert 'hello' + ' ' + 'world' == 'hello world', 'basic concat'
assert '' + 'test' == 'test', 'empty left concat'
assert 'test' + '' == 'test', 'empty right concat'
assert '' + '' == '', 'empty both concat'
assert 'a' + 'b' + 'c' + 'd' == 'abcd', 'multiple concat'

# === Augmented assignment (+=) ===
s = 'hello'
s += ' world'
assert s == 'hello world', 'basic iadd'

s = 'test'
s += ''
assert s == 'test', 'iadd empty'

s = 'a'
s += 'b'
s += 'c'
assert s == 'abc', 'multiple iadd'

s = 'ab'
s += s
assert s == 'abab', 'iadd self'

# === String length ===
assert len('') == 0, 'len empty'
assert len('a') == 1, 'len single'
assert len('hello') == 5, 'len basic'
assert len('hello world') == 11, 'len with space'
assert len('caf\xe9') == 4, 'len unicode'

# === String repr/str ===
assert repr('') == "''", 'empty string repr'
assert str('') == '', 'empty string str'

assert repr('hello') == "'hello'", 'string repr'
assert str('hello') == 'hello', 'string str'

assert repr('hello "world"') == '\'hello "world"\'', 'string with quotes repr'
assert str('hello "world"') == 'hello "world"', 'string with quotes str'

# === String repetition (*) ===
assert 'ab' * 3 == 'ababab', 'str mult int'
assert 3 * 'ab' == 'ababab', 'int mult str'
assert 'x' * 0 == '', 'str mult zero'
assert 'x' * -1 == '', 'str mult negative'
assert '' * 5 == '', 'empty str mult'
assert 'a' * 1 == 'a', 'str mult one'

# === String repetition augmented assignment (*=) ===
s = 'ab'
s *= 3
assert s == 'ababab', 'str imult'

s = 'x'
s *= 0
assert s == '', 'str imult zero'

# === String join method ===
# Basic join on literals
assert ','.join(['a', 'b', 'c']) == 'a,b,c', 'join list with comma'
assert ''.join(['a', 'b', 'c']) == 'abc', 'join with empty separator'
assert '-'.join([]) == '', 'join empty list'
assert ','.join(['only']) == 'only', 'join single element'

# Join with different iterables
assert ' '.join(('hello', 'world')) == 'hello world', 'join tuple'

# Join with string iterable (iterates over characters)
assert ','.join('abc') == 'a,b,c', 'join string iterable'

# Join with variable separator
sep = '-'
assert sep.join(['a', 'b']) == 'a-b', 'join with variable separator'

# Heap-allocated string separator
s = str('.')
assert s.join(['a', 'b']) == 'a.b', 'join with heap string'

# Mixed string types in iterable (interned and heap)
mixed = ['hello', str('world')]
assert ' '.join(mixed) == 'hello world', 'join with mixed string types'

# === String indexing (getitem) ===
# Basic indexing
assert 'hello'[0] == 'h', 'getitem index 0'
assert 'hello'[1] == 'e', 'getitem index 1'
assert 'hello'[4] == 'o', 'getitem last index'

# Negative indexing
assert 'hello'[-1] == 'o', 'getitem -1'
assert 'hello'[-2] == 'l', 'getitem -2'
assert 'hello'[-5] == 'h', 'getitem -5'

# Single character strings
assert 'a'[0] == 'a', 'getitem single char at 0'
assert 'a'[-1] == 'a', 'getitem single char at -1'

# Unicode strings
s = 'café'
assert s[0] == 'c', 'unicode getitem 0'
assert s[1] == 'a', 'unicode getitem 1'
assert s[2] == 'f', 'unicode getitem 2'
assert s[3] == 'é', 'unicode getitem 3 (accented)'
assert s[-1] == 'é', 'unicode getitem -1'

# Multi-byte unicode (CJK characters)
s = '日本語'
assert s[0] == '日', 'cjk getitem 0'
assert s[1] == '本', 'cjk getitem 1'
assert s[2] == '語', 'cjk getitem 2'
assert s[-1] == '語', 'cjk getitem -1'

# Emoji (multi-byte UTF-8)
s = 'a🎉b'
assert s[0] == 'a', 'emoji string getitem 0'
assert s[1] == '🎉', 'emoji string getitem 1 (emoji)'
assert s[2] == 'b', 'emoji string getitem 2'

# Heap-allocated strings
s = str('hello')
assert s[0] == 'h', 'heap string getitem'
assert s[-1] == 'o', 'heap string negative getitem'

# Variable index
s = 'abc'
i = 1
assert s[i] == 'b', 'getitem with variable index'

# Bool indices (True=1, False=0)
s = 'abc'
assert s[False] == 'a', 'str getitem with False'
assert s[True] == 'b', 'str getitem with True'
